searchData={"items":[{"type":"module","title":"Tempus","doc":"`Tempus` is a library to deal with timeslots.\n\nIt aims to be a fast yet easy to use implementation of a schedule of any type,\n  including but not limited to free/busy time schedules.\n\nThe example of it might be a calendar software, where slots might be marked as\n  free, or busy. It also allows simple arithmetics with schedules, like adding\n  five days or subtracting 7 hours 30 minutes from now, considering busy slots.","ref":"Tempus.html"},{"type":"function","title":"Tempus.add/4","doc":"Adds an amount of units to the origin, considering slots given.","ref":"Tempus.html#add/4"},{"type":"function","title":"Examples - Tempus.add/4","doc":"iex> slots = [\n    ...>   ~D|2020-08-07|,\n    ...>   ~D|2020-08-10|,\n    ...>   ~D|2020-08-11|,\n    ...>   ~D|2020-08-14|\n    ...> ] |> Enum.into(Tempus.Slots.new(:stream, []))\n    iex> Tempus.add(slots, ~U|2020-08-11 23:00:00Z|, 0, :second)\n    ~U[2020-08-12 00:00:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-12 01:00:00Z|, 0, :second)\n    ~U[2020-08-12 01:00:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-11 23:00:00Z|, -10*60+1, :second)\n    ~U[2020-08-09 23:50:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-12 00:09:00Z|, -10*60, :second)\n    ~U[2020-08-09 23:59:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-12 00:10:00Z|, -10*60, :second)\n    ~U[2020-08-12 00:00:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-12 00:10:00Z|, -10*60-1, :second)\n    ~U[2020-08-09 23:59:59Z]\n    iex> Tempus.add(slots, ~U|2020-08-11 23:00:00Z|, 10*60, :second)\n    ~U[2020-08-12 00:10:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-12 00:00:00Z|, 10*60, :second)\n    ~U[2020-08-12 00:10:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-09 23:55:00Z|, 10*60, :second)\n    ~U[2020-08-12 00:05:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-08 23:55:00Z|, 10*60, :second)\n    ~U[2020-08-09 00:05:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-06 23:55:00Z|, 2*3600*24 + 10*60, :second)\n    ~U[2020-08-12 00:05:00Z]\n\n    iex> slots = Tempus.Slots.new(:stream, [])\n    iex> Tempus.add(slots, ~U|2020-08-12 01:00:00Z|, 0, :second)\n    ~U[2020-08-12 01:00:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-11 23:00:00Z|, 5*60+1, :second)\n    ~U[2020-08-11 23:05:01.000000Z]\n    iex> Tempus.add(slots, ~U|2020-08-11 23:00:00Z|, -10*60+1, :second)\n    ~U[2020-08-11 22:50:01Z]\n\n    iex> slots = Tempus.Slots.new(:list, [])\n    iex> Tempus.add(slots, ~U|2020-08-12 01:00:00Z|, 0, :second)\n    ~U[2020-08-12 01:00:00Z]\n    iex> Tempus.add(slots, ~U|2020-08-11 23:00:00Z|, 5*60+1, :second)\n    ~U[2020-08-11 23:05:01Z]\n    iex> Tempus.add(slots, ~U|2020-08-11 23:00:00Z|, -10*60+1, :second)\n    ~U[2020-08-11 22:50:01Z]\n    iex> slots |> Tempus.add(1) |> DateTime.to_date()\n    Date.utc_today()","ref":"Tempus.html#add/4-examples"},{"type":"function","title":"Tempus.days_add/2","doc":"Returns the reversed list of free days after origin.","ref":"Tempus.html#days_add/2"},{"type":"function","title":"Examples - Tempus.days_add/2","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> Tempus.days_add(slots, origin: ~D|2020-08-07|, count: 3) |> hd()\n    ~D|2020-08-12|\n    iex> Tempus.days_add(slots, origin: ~D|2020-08-07|, count: 3, direction: :fwd) |> hd()\n    ~D|2020-08-12|\n    iex> Tempus.days_add(slots, origin: ~D|2020-08-07|, count: -3, direction: :bwd) |> hd()\n    ~D|2020-08-12|\n    iex> Tempus.days_add(slots, origin: ~D|2020-08-12|, count: -4) |> hd()\n    ~D|2020-08-06|\n    iex> Tempus.days_add(slots, origin: ~D|2020-08-12|, count: 4, direction: :bwd) |> hd()\n    ~D|2020-08-06|\n    iex> Tempus.days_add(slots, origin: ~D|2020-08-12|, count: -4, direction: :fwd) |> hd()\n    ~D|2020-08-06|","ref":"Tempus.html#days_add/2-examples"},{"type":"function","title":"Tempus.days_ago/3","doc":"Returns the reversed list of free days after origin.","ref":"Tempus.html#days_ago/3"},{"type":"function","title":"Examples - Tempus.days_ago/3","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> Tempus.days_ago(slots, ~D|2020-08-07|, 0)\n    [~D|2020-08-06|]\n    iex> Tempus.days_ago(slots, ~D|2020-08-12|, 4) |> hd()\n    ~D|2020-08-06|","ref":"Tempus.html#days_ago/3-examples"},{"type":"function","title":"Tempus.days_ahead/3","doc":"Returns the reversed list of free days after origin.","ref":"Tempus.html#days_ahead/3"},{"type":"function","title":"Examples - Tempus.days_ahead/3","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> Tempus.days_ahead(slots, ~D|2020-08-07|, 0)\n    [~D|2020-08-08|]\n    iex> Tempus.days_ahead(slots, ~D|2020-08-07|, 3) |> hd()\n    ~D|2020-08-12|","ref":"Tempus.html#days_ahead/3-examples"},{"type":"function","title":"Tempus.drop_while/2","doc":"Drops slots at the beginning of the `%Slots{}` struct while `fun` returns a truthy value.","ref":"Tempus.html#drop_while/2"},{"type":"function","title":"Examples - Tempus.drop_while/2","doc":"iex> import Tempus.Guards\n    ...> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> slots\n    ...> |> Tempus.drop_while(&is_slot_coming_before(&1, Tempus.Slot.wrap(~D|2020-08-09|)))\n    ...> |> Enum.count()\n    1","ref":"Tempus.html#drop_while/2-examples"},{"type":"function","title":"Tempus.free?/2","doc":"Checks whether the slot is disjoined against slots.","ref":"Tempus.html#free?/2"},{"type":"function","title":"Examples - Tempus.free?/2","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> Tempus.free?(slots, ~D|2020-08-07|)\n    false\n    iex> Tempus.free?(slots, ~D|2020-08-08|)\n    true\n    iex> Tempus.free?(slots, ~U|2020-08-09T23:59:59.999999Z|)\n    true\n    iex> Tempus.free?(slots, DateTime.add(~U|2020-08-09T23:59:59.999999Z|, 1, :microsecond))\n    false","ref":"Tempus.html#free?/2-examples"},{"type":"function","title":"Tempus.guess/1","doc":"Helper to instantiate slot from any known format, by wrapping the argument.","ref":"Tempus.html#guess/1"},{"type":"function","title":"Examples - Tempus.guess/1","doc":"iex> Tempus.guess(\"2023-04-10\")\n    {:ok, Tempus.Slot.wrap(~D[2023-04-10])}\n    iex> Tempus.guess(nil)\n    {:ok, %Tempus.Slot{from: nil, to: nil}}\n    iex> Tempus.guess(\"∞\")\n    {:ok, %Tempus.Slot{from: nil, to: nil}}\n    iex> Tempus.guess(\"\")\n    {:ok, %Tempus.Slot{from: nil, to: nil}}\n    iex> Tempus.guess(\"2023-04-10T10:00:00Z\")\n    {:ok, Tempus.Slot.wrap(~U[2023-04-10T10:00:00Z])}\n    iex> Tempus.guess(\"10:00:00\")\n    {:ok, Date.utc_today() |> DateTime.new!(Time.from_erl!({10, 0, 0})) |> Tempus.Slot.wrap()}\n    iex> Tempus.guess(\"20230410T235007.123+0230\")\n    if Version.compare(System.version(), \"1.14.0\") == :lt do\n      {:error, :invalid_format}\n    else\n      {:ok, Tempus.Slot.wrap(~U[2023-04-10T21:20:07.123Z])}\n    end\n    iex> Tempus.guess(\"2023-04-10-15\")\n    {:error, :invalid_format}","ref":"Tempus.html#guess/1-examples"},{"type":"function","title":"Tempus.guess/2","doc":"Helper to instantiate slot from any known format, by joining the arguments.","ref":"Tempus.html#guess/2"},{"type":"function","title":"Examples - Tempus.guess/2","doc":"iex> import Tempus.Sigils\n    iex> Tempus.guess(\"2023-04-10\", \"2023-04-12\")\n    {:ok, ~I[2023-04-10 00:00:00.000000Z|2023-04-12 23:59:59.999999Z]}\n    iex> Tempus.guess(\"2023-04-10\", nil)\n    {:ok, ~I[2023-04-10 00:00:00.000000Z|2023-04-10T23:59:59.999999Z]}\n    iex> Tempus.guess(\"2023-04-10T10:00:00Z\", \"2023-04-12\")\n    {:ok, ~I[2023-04-10 10:00:00Z|2023-04-12 23:59:59.999999Z]}\n    iex> Tempus.guess(\"20230410T235007.123+0230\", \"2023-04-12\")\n    if Version.compare(System.version(), \"1.14.0\") == :lt do\n      {:error, {:invalid_arguments, [from: :invalid_format]}}\n    else\n      {:ok, ~I[2023-04-10 21:20:07.123Z|2023-04-12 23:59:59.999999Z]}\n    end\n    iex> Tempus.guess(\"2023-04-10\", :ok)\n    {:error, {:invalid_arguments, [to: :invalid_argument]}}\n    iex> Tempus.guess(:ok, \"2023-04-10\")\n    {:error, {:invalid_arguments, [from: :invalid_argument]}}\n    iex> Tempus.guess(\"2023-04-10-15\", :ok)\n    {:error, {:invalid_arguments, [from: :invalid_format, to: :invalid_argument]}}\n    iex> Tempus.guess(\"2023-20-40\", \"10:70:80\")\n    {:error, {:invalid_arguments, [from: :invalid_date, to: :invalid_time]}}","ref":"Tempus.html#guess/2-examples"},{"type":"function","title":"Tempus.new/1","doc":"Creates the new instance of `Tempus.Slots.t(Tempus.Slots.Stream.t())` backed by stream.\n\nThis is a convinience wrapper for `Tempus.Slots.new(:stream, data)`.","ref":"Tempus.html#new/1"},{"type":"function","title":"Tempus.next_busy/2","doc":"Returns the next **busy** slot from the slots passed as a first argument,\n  that immediately follows `origin`. If slots are overlapped, the overlapped\n  one gets returned.","ref":"Tempus.html#next_busy/2"},{"type":"function","title":"Examples - Tempus.next_busy/2","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> Tempus.next_busy(slots, origin: %Tempus.Slot{from: ~U|2020-08-08 23:00:00Z|, to: ~U|2020-08-09 12:00:00Z|})\n    %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: %Tempus.Slot{from: ~U|2020-08-07 11:00:00Z|, to: ~U|2020-08-07 12:00:00Z|}, count: 2) |> hd()\n    %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: %Tempus.Slot{from: ~U|2020-08-07 11:00:00Z|, to: ~U|2020-08-08 12:00:00Z|})\n    %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: %Tempus.Slot{from: ~U|2020-08-07 11:00:00Z|, to: ~U|2020-08-10 12:00:00Z|})\n    %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-07|)\n    %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-08|)\n    %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-08|, direction: :bwd)\n    %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-10|, direction: :bwd)\n    %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-10|, direction: :bwd, count: :infinity)\n    slots\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-10|, count: 3.1415)\n    ** (Tempus.ArgumentError) invalid argument: expected ‹Elixir.Integer›, got: ‹3.1415›\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-10|, direction: :bwd, count: 2)\n    Enum.to_list(slots)\n    iex> Tempus.next_busy(slots, origin: ~D|2020-08-10|, direction: :bwd, count: 1)\n    Enum.drop(slots, 1)\n    iex> Tempus.next_busy(slots, origin: %Tempus.Slot{from: ~U|2020-08-11 11:00:00Z|, to: ~U|2020-08-11 12:00:00Z|})\n    nil\n    iex> Tempus.next_busy(slots, origin: %Tempus.Slot{from: ~U|2020-08-06 11:00:00Z|, to: ~U|2020-08-06 12:00:00Z|}, direction: :bwd)\n    nil\n    iex> Tempus.next_busy(%Tempus.Slots{})\n    nil","ref":"Tempus.html#next_busy/2-examples"},{"type":"function","title":"Tempus.next_free/2","doc":"Returns the next **free** slot from the slots passed as a first argument,\n  that immediately follows `origin`. If slots are overlapped, the overlapped\n  one gets returned.","ref":"Tempus.html#next_free/2"},{"type":"function","title":"Examples - Tempus.next_free/2","doc":"iex> import Tempus.Sigils\n    iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-12|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-14|),\n    ...>   Tempus.Slot.wrap(~D|2030-08-14|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> Tempus.next_free(slots, origin: %Tempus.Slot{from: ~U|2020-08-08 23:00:00Z|, to: ~U|2020-08-09 12:00:00Z|})\n    ~I[2020-08-08 00:00:00.000000Z → 2020-08-09 23:59:59.999999Z]\n    iex> Tempus.next_free(slots, origin: %Tempus.Slot{from: ~U|2020-08-06 11:00:00Z|, to: ~U|2020-08-06 12:00:00Z|})\n    ~I[∞ → 2020-08-06 23:59:59.999999Z]nu\n    iex> Tempus.next_free(slots, origin: ~U|2020-08-13 01:00:00.000000Z|)\n    ~I[2020-08-13 00:00:00.000000Z → 2020-08-13 23:59:59.999999Z]\n    iex> Tempus.next_free(slots, origin: ~D|2020-08-13|)\n    ~I[2020-08-13 00:00:00.000000Z → 2020-08-13 23:59:59.999999Z]\n    iex> Tempus.next_free(slots, origin: ~D|2020-08-14|)\n    ~I[2020-08-15 00:00:00.000000Z → 2030-08-13 23:59:59.999999Z]\n    iex> Tempus.next_free(slots)\n    ~I[2020-08-15 00:00:00.000000Z → 2030-08-13 23:59:59.999999Z]","ref":"Tempus.html#next_free/2-examples"},{"type":"function","title":"Tempus.parse_cron/2","doc":"Creates a `Tempus.Slots.Stream.t()` slots stream by a cron-like definition.","ref":"Tempus.html#parse_cron/2"},{"type":"function","title":"Examples - Tempus.parse_cron/2","doc":"iex> import Tempus.Sigils\n    iex> ~U[2024-06-07 12:00:00Z] |> Tempus.parse_cron(\"10-30/15 */4 1 */1 6,7\") |> Enum.take(5)\n    [\n      ~I(2024-06-08T00:10:00Z → 2024-06-08T00:10:00Z),\n      ~I(2024-06-08T00:25:00Z → 2024-06-08T00:25:00Z),\n      ~I(2024-06-08T04:10:00Z → 2024-06-08T04:10:00Z),\n      ~I(2024-06-08T04:25:00Z → 2024-06-08T04:25:00Z),\n      ~I(2024-06-08T08:10:00Z → 2024-06-08T08:10:00Z)\n    ]","ref":"Tempus.html#parse_cron/2-examples"},{"type":"function","title":"Tempus.slice/4","doc":"Slices the `%Slots{}` based on origins `from` and `to` and an optional type\n  (default: `:reluctant`.) Returns sliced `%Slots{}` back.","ref":"Tempus.html#slice/4"},{"type":"function","title":"Examples - Tempus.slice/4","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> slots |> Tempus.slice(~D|2020-08-07|, ~D|2020-08-11|) |> Enum.count()\n    1","ref":"Tempus.html#slice/4-examples"},{"type":"function","title":"Tempus.slot/2","doc":"","ref":"Tempus.html#slot/2"},{"type":"function","title":"Tempus.slot!/2","doc":"","ref":"Tempus.html#slot!/2"},{"type":"function","title":"Tempus.slots/1","doc":"Syntactic sugar for `|> Enum.into(%Slots{})`.","ref":"Tempus.html#slots/1"},{"type":"function","title":"Examples - Tempus.slots/1","doc":"iex> [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-12|)\n    ...> ] |> Tempus.slots()\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n        %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]},\n        %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]},\n        %Tempus.Slot{from: ~U[2020-08-12 00:00:00.000000Z], to: ~U[2020-08-12 23:59:59.999999Z]}]}}","ref":"Tempus.html#slots/1-examples"},{"type":"function","title":"Tempus.stream/2","doc":"Syntactic sugar for `Tempus.Slots.Stream.iterate/3` with default options `return_as: :slots, join: true`.","ref":"Tempus.html#stream/2"},{"type":"function","title":"Examples - Tempus.stream/2","doc":"iex> import Tempus.Sigils\n    ...> Tempus.stream(~D|2024-01-20|, &Tempus.Slot.shift(&1, by: rem(&1.from.day, 2) + 1, unit: :day)) |> Enum.take(3)\n    [~I(2024-01-20T00:00:00.000000Z → 2024-01-21T23:59:59.999999Z),\n     ~I(2024-01-23T00:00:00.000000Z → 2024-01-23T23:59:59.999999Z),\n     ~I(2024-01-25T00:00:00.000000Z → 2024-01-25T23:59:59.999999Z)]","ref":"Tempus.html#stream/2-examples"},{"type":"function","title":"Tempus.take/2","doc":"Takes an amount of elements from the beginning of the `Tempus.Slots` structure, returning `Tempus.Slots.List` instance.\n\nIf amount is 0, it returns `Tempus.Slots.List.new/0`.","ref":"Tempus.html#take/2"},{"type":"function","title":"Examples - Tempus.take/2","doc":"iex> import Tempus.Sigils\n    ...> Tempus.stream(~D|2024-01-20|, &Tempus.Slot.shift(&1, by: rem(&1.from.day, 2) + 1, unit: :day)) |> Tempus.take(3)\n    Tempus.Slots.new(:list, [~I(2024-01-20T00:00:00.000000Z → 2024-01-21T23:59:59.999999Z),\n                             ~I(2024-01-23T00:00:00.000000Z → 2024-01-23T23:59:59.999999Z),\n                             ~I(2024-01-25T00:00:00.000000Z → 2024-01-25T23:59:59.999999Z)])","ref":"Tempus.html#take/2-examples"},{"type":"function","title":"Tempus.take_while/2","doc":"Takes slots at the beginning of the `%Slots{}` struct while `fun` returns a truthy value.","ref":"Tempus.html#take_while/2"},{"type":"function","title":"Examples - Tempus.take_while/2","doc":"iex> import Tempus.Guards\n    ...> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    iex> slots\n    ...> |> Tempus.take_while(&is_slot_coming_before(&1, Tempus.Slot.wrap(~D|2020-08-09|)))\n    ...> |> Enum.count()\n    1","ref":"Tempus.html#take_while/2-examples"},{"type":"type","title":"Tempus.count/0","doc":"Number of slots (`:stream` means lazy folding, unknown upfront)","ref":"Tempus.html#t:count/0"},{"type":"type","title":"Tempus.direction/0","doc":"Direction for slots navigation","ref":"Tempus.html#t:direction/0"},{"type":"type","title":"Tempus.option/0","doc":"Navigation option","ref":"Tempus.html#t:option/0"},{"type":"type","title":"Tempus.options/0","doc":"Argument containing navigation options","ref":"Tempus.html#t:options/0"},{"type":"type","title":"Tempus.slice_type/0","doc":"The type defining how slicing is to be applied.\nWhen `:greedy`, overlapping boundary slots would be included,\n  `:reluctant` would take only those fully contained in the interval.","ref":"Tempus.html#t:slice_type/0"},{"type":"behaviour","title":"Tempus.Slots.Behaviour","doc":"The bunch of functions expected to be implemented by `Tempus.Slots` implementations.","ref":"Tempus.Slots.Behaviour.html"},{"type":"callback","title":"Tempus.Slots.Behaviour.new/0","doc":"Creates a new instance, which is also usually an identity element","ref":"Tempus.Slots.Behaviour.html#c:new/0"},{"type":"protocol","title":"Tempus.Slots.Group","doc":"The protocol to implement for the ordered collection of slots.","ref":"Tempus.Slots.Group.html"},{"type":"function","title":"Tempus.Slots.Group.add/3","doc":"Adds a single `t:Slot.t/0` instance to this `t:Slots.t/1` (which is effectively `Slots.t(t())`)\n  implementation. If `merge/3` implementation returns `{:error, __MODULE__}`,\n  this function would be used to add elements one by one through `reduce/3`.","ref":"Tempus.Slots.Group.html#add/3"},{"type":"function","title":"Tempus.Slots.Group.flatten/2","doc":"Flattens the implementation, returning the list back.","ref":"Tempus.Slots.Group.html#flatten/2"},{"type":"function","title":"Tempus.Slots.Group.identity/1","doc":"The function returning the identity element, which can be merged into\neach and every implementation without changing it.","ref":"Tempus.Slots.Group.html#identity/1"},{"type":"function","title":"Tempus.Slots.Group.inverse/2","doc":"Efficient implementation of inversing slots. If this function\n  returns `{:error, __MODULE__}`, the `reduce/3` will be used instead.","ref":"Tempus.Slots.Group.html#inverse/2"},{"type":"function","title":"Tempus.Slots.Group.merge/3","doc":"Efficient implementation of merging slots. If this function\n  returns `{:error, __MODULE__}`, the `reduce/3` will be used instead.","ref":"Tempus.Slots.Group.html#merge/3"},{"type":"function","title":"Tempus.Slots.Group.split/3","doc":"Efficient implementation of splitting slots. If this function\n  returns `{:error, __MODULE__}`, the `reduce/3` will be used instead.","ref":"Tempus.Slots.Group.html#split/3"},{"type":"type","title":"Tempus.Slots.Group.t/0","doc":"All the types that implement this protocol.","ref":"Tempus.Slots.Group.html#t:t/0"},{"type":"module","title":"Tempus.Crontab","doc":"Helper functions to work with `cron` syntax.","ref":"Tempus.Crontab.html"},{"type":"function","title":"Tempus.Crontab.formula/1","doc":"Produces the single formula out of cron record. Might be useful\nfor some external check that requires the single validation call.\n\n_Examples_\n\n    iex> Tempus.Crontab.formula(\"42 3 28 08 *\").formula |> String.split(\" && \") |> Enum.sort()\n    [\"(day == 28)\", \"(hour == 3)\", \"(minute == 42)\", \"(month == 8)\", \"(rem(day_of_week, 1) == 0)\"]\n\n    iex> Tempus.Crontab.formula(\"423 * * * *\")\n    {:error, [minute: {:could_not_parse_field, [\"423\"]}]}","ref":"Tempus.Crontab.html#formula/1"},{"type":"function","title":"Tempus.Crontab.next/3","doc":"Returns the next `DateTime` the respective `cron` record points to\nwith a precision given as the third argument (default: `:second`.)\n\nIf the first parameter is not given, it assumes _the next after now_.\n\n_Examples_\n\n    iex> dt = DateTime.from_unix!(1567091960)\n    ~U[2019-08-29 15:19:20Z]\n    iex> Tempus.Crontab.next(dt, \"42 3 28 08 *\")\n    [\n      origin: ~U[2019-08-29 15:19:20Z],\n      next: ~U[2020-08-28 03:42:00Z],\n      second: 31494160\n    ]\n\nwhere `origin` contains the timestamp to lookup the `next` for, `next`\nis the `DateTime` instance of the next event and `second` is the\n{`precision`, `difference_in_that_precision`}.","ref":"Tempus.Crontab.html#next/3"},{"type":"function","title":"Tempus.Crontab.next_as_list/3","doc":"Returns the _list_ of all the events after `dt` (default: `DateTime.utc_now/0`.)\n\nThis function calculates the outcome greedily and, while it might be slightly\nfaster than `Tempus.Crontab.next_as_stream/3`, it should not be used for\nfrequently recurring cron records (like `\"* * * * *\"`.)","ref":"Tempus.Crontab.html#next_as_list/3"},{"type":"function","title":"Tempus.Crontab.next_as_stream/3","doc":"Returns the _stream_ of all the events after `dt` (default: `DateTime.utc_now/0`.)\n\nThis function calculates the outcome lazily, returning a stream.\n\nSee `Tempus.Crontab.next_as_list/3` for greedy evaluation.","ref":"Tempus.Crontab.html#next_as_stream/3"},{"type":"function","title":"Examples - Tempus.Crontab.next_as_stream/3","doc":"iex> ~U[2024-06-07 12:00:00Z] |> Tempus.Crontab.next_as_stream(\"10-30/5 */4 1 */1 6,7\") |> Enum.take(2)\n    [\n      [origin: ~U[2024-06-07 12:00:00Z], next: ~U[2024-06-08 00:10:00Z], second: 43800],\n      [origin: ~U[2024-06-07 12:00:00Z], next: ~U[2024-06-08 00:15:00Z], second: 44100]\n    ]","ref":"Tempus.Crontab.html#next_as_stream/3-examples"},{"type":"function","title":"Tempus.Crontab.parse/1","doc":"Parses the cron string into human-readable representation.\n\n**This function is exported for debugging purposes only, normally one would call `prepare/1` instead.**\n\nInput format: [\"minute hour day/month month day/week\"](https://crontab.guru/).\n\n_Examples:_\n\n    iex> Tempus.Crontab.parse \"10-30/5 */4 1 */1 6,7\"\n    %Tempus.Crontab{\n      day: \"(day == 1)\",\n      day_of_week: \"(day_of_week == 6 || day_of_week == 7)\",\n      hour: \"(rem(hour, 4) == 0)\",\n      minute: \"(rem(minute, 5) == 0 && minute >= 10 && minute <= 30)\",\n      month: \"(rem(month, 1) == 0)\"\n    }\n\n_In case of malformed input:_\n\n    iex> Tempus.Crontab.parse \"10-30/5 */4 1 */1 6d,7\"\n    %Tempus.Crontab{\n      day: \"(day == 1)\",\n      day_of_week: {:error, {:could_not_parse_integer, \"6d\"}},\n      hour: \"(rem(hour, 4) == 0)\",\n      minute: \"(rem(minute, 5) == 0 && minute >= 10 && minute <= 30)\",\n      month: \"(rem(month, 1) == 0)\"\n    }","ref":"Tempus.Crontab.html#parse/1"},{"type":"function","title":"Tempus.Crontab.prepare/1","doc":"Parses the cron string into `Tempus.Crontab.t()` struct.\n\nInput format: [\"minute hour day/month month day/week\"](https://crontab.guru/).","ref":"Tempus.Crontab.html#prepare/1"},{"type":"function","title":"Tempus.Crontab.to_cron/1","doc":"Converts the `Time` instance into daily-execution cron string","ref":"Tempus.Crontab.html#to_cron/1"},{"type":"type","title":"Tempus.Crontab.t/0","doc":"Internal representation of the record in cron file","ref":"Tempus.Crontab.html#t:t/0"},{"type":"module","title":"Tempus.Guards","doc":"Handy guards to simplify pattern matching slots","ref":"Tempus.Guards.html"},{"type":"macro","title":"Tempus.Guards.is_coming_before/2","doc":"Guard to compare two instances of `t:Tempus.Slot.origin/0`.\n\nBeware this guards returns a meaningful `true`/`false` if and only all the input types\n  are correct. When passing not `t:Slot.origin/0` as any of parameter it would\n  return `false` and therefore it’s negation is tricky.","ref":"Tempus.Guards.html#is_coming_before/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_coming_before/2","doc":"iex> import Tempus.Guards, only: [is_coming_before: 2]\n    ...> is_coming_before(~D[2023-04-10], ~U[2023-04-11T00:00:00.000000Z])\n    true\n    iex> is_coming_before(~D[2023-04-10], ~D[2023-04-10])\n    false\n    iex> s_bcn = ~U[2023-06-26T09:30:00Z]\n    ...> s_ny = ~D|2024-06-26| |> Tempus.Slot.wrap() |> Tempus.Slot.shift_tz(\"America/New_York\")\n    ...> is_coming_before(s_bcn, s_ny)\n    true","ref":"Tempus.Guards.html#is_coming_before/2-examples"},{"type":"macro","title":"Tempus.Guards.is_coming_before_in_ms/2","doc":"For any calendar, checks if the former argument comes before the latter one.\n\nAllowed in guard tests. Inlined by the compiler.","ref":"Tempus.Guards.html#is_coming_before_in_ms/2"},{"type":"macro","title":"Tempus.Guards.is_covered/2","doc":"Guard to validate the slot covers the origin passed as the first argument","ref":"Tempus.Guards.html#is_covered/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_covered/2","doc":"iex> import Tempus.Guards, only: [is_covered: 2]\n    ...> import Tempus.Sigils\n    ...> {from, to} = {~U[2023-04-10 00:00:00Z], ~U[2023-04-10 00:59:59Z]}\n    ...> s = %Tempus.Slot{from: from, to: to}\n    ...> is_covered(from, s) and is_covered(to, s)\n    true\n    iex> s1 = ~I[2023-04-10 00:00:00Z|2023-04-11 00:00:00Z]\n    ...> s2 = Tempus.Slot.wrap(~D|2023-04-10|)\n    ...> is_covered(s1, s2)\n    false\n    iex> s1 = ~I[2023-04-10 00:00:00Z|2023-04-11 00:00:00Z]\n    ...> s2 = Tempus.Slot.wrap(~D|2023-04-10|)\n    ...> is_covered(s1, s2)\n    false\n    iex> s_bcn = ~U[2023-06-26T09:30:00Z]\n    ...> s_ny = ~D|2023-06-26| |> Tempus.Slot.wrap() |> Tempus.Slot.shift_tz(\"America/New_York\")\n    ...> is_covered(s_bcn, s_ny)\n    true\n    iex> s_bcn = ~D[2023-06-26]\n    ...> s_ny = Tempus.slot!(~D|2023-06-26|, ~D|2023-06-27|) |> Tempus.Slot.shift_tz(\"America/New_York\")\n    ...> is_covered(s_bcn, s_ny)\n    false","ref":"Tempus.Guards.html#is_covered/2-examples"},{"type":"macro","title":"Tempus.Guards.is_date/1","doc":"Syntactic sugar for `is_struct(term, Date)`.","ref":"Tempus.Guards.html#is_date/1"},{"type":"macro","title":"Tempus.Guards.is_datetime/1","doc":"Syntactic sugar for `is_struct(term, DateTime)`.","ref":"Tempus.Guards.html#is_datetime/1"},{"type":"macro","title":"Tempus.Guards.is_datetime_coming_before/2","doc":"Guard to validate if two `DateTime` instances passed as parameters if the former one\n  is less (comes earlier) than the latter one.\n\nIf arguments have different timezones, returns a meaningful value for positive unix epoch only.\n\nIf arguments are not datetimes, or timezones differ and values are before epoch, returns `false`.","ref":"Tempus.Guards.html#is_datetime_coming_before/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_datetime_coming_before/2","doc":"iex> import Tempus.Guards, only: [is_datetime_coming_before: 2]\n    ...> is_datetime_coming_before(~U|2020-01-01T12:00:00Z|,\n    ...>     DateTime.shift_zone!(~U|2020-01-01T12:00:00Z|, \"Australia/Sydney\"))\n    false\n    iex> is_datetime_coming_before(~U|2000-01-01T12:00:00Z|, ~U|2000-01-01T23:59:59Z|)\n    true","ref":"Tempus.Guards.html#is_datetime_coming_before/2-examples"},{"type":"macro","title":"Tempus.Guards.is_datetime_covered/2","doc":"Guard to validate if the `DateTime` instance passed as a first argument is covered by the slot.","ref":"Tempus.Guards.html#is_datetime_covered/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_datetime_covered/2","doc":"iex> import Tempus.Guards, only: [is_datetime_covered: 2]\n    ...> slot = Tempus.slot!(~U|2020-01-01T12:00:00Z|, ~U|2020-01-01T23:59:59Z|)\n    ...> is_datetime_covered(~U|2020-01-01T18:00:00Z|, slot)\n    true\n    ...> is_datetime_covered(~U|2020-01-01T06:00:00Z|, slot)\n    false","ref":"Tempus.Guards.html#is_datetime_covered/2-examples"},{"type":"macro","title":"Tempus.Guards.is_datetime_equal/2","doc":"Guard to validate if two `DateTime` instances passed as parameters for pointing to the same `DateTime`.\nIf arguments have different timezones, returns a meaningful value for positive unix epoch only.","ref":"Tempus.Guards.html#is_datetime_equal/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_datetime_equal/2","doc":"iex> import Tempus.Guards, only: [is_datetime_equal: 2]\n    ...> is_datetime_equal(~U|2020-01-01T12:00:00Z|,\n    ...>     DateTime.shift_zone!(~U|2020-01-01T12:00:00Z|, \"Australia/Sydney\"))\n    true\n    iex> is_datetime_equal(~U|2000-01-01T12:00:00Z|, ~U|2000-01-01T23:59:59Z|)\n    false","ref":"Tempus.Guards.html#is_datetime_equal/2-examples"},{"type":"macro","title":"Tempus.Guards.is_equal_in_ms/2","doc":"For any calendar, checks if the former argument comes before the latter one.\n\nAllowed in guard tests. Inlined by the compiler.","ref":"Tempus.Guards.html#is_equal_in_ms/2"},{"type":"macro","title":"Tempus.Guards.is_joint/2","doc":"Guard to validate one slot ovelaps another","ref":"Tempus.Guards.html#is_joint/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_joint/2","doc":"iex> import Tempus.Guards, only: [is_joint: 2]\n    ...> import Tempus.Sigils\n    ...> s1 = ~I[2023-04-09 23:00:00Z|2023-04-10 00:59:59Z]\n    ...> s2 = Tempus.Slot.wrap(~D|2023-04-10|)\n    ...> is_joint(s1, s2)\n    true\n    iex> s1 = ~I[2023-04-09 23:00:00Z|2023-04-10 00:00:00Z]\n    ...> s2 = Tempus.Slot.wrap(~D|2023-04-10|)\n    ...> is_joint(s1, s2)\n    true","ref":"Tempus.Guards.html#is_joint/2-examples"},{"type":"macro","title":"Tempus.Guards.is_leap/1","doc":"Returns `true` if the year is leap, and `false` otherwise.","ref":"Tempus.Guards.html#is_leap/1"},{"type":"macro","title":"Examples - Tempus.Guards.is_leap/1","doc":"iex> import Tempus.Guards, only: [is_leap: 1]\n    ...> is_leap(1970)\n    false\n    ...> is_leap(2000)\n    true\n\nAllowed in guard tests. Inlined by the compiler.","ref":"Tempus.Guards.html#is_leap/1-examples"},{"type":"macro","title":"Tempus.Guards.is_locator/1","doc":"Guard to validate that the term given is actually a `t:Tempus.Slot.origin/0` _or_\n  a function which might be used as a slot locator.","ref":"Tempus.Guards.html#is_locator/1"},{"type":"macro","title":"Examples - Tempus.Guards.is_locator/1","doc":"iex> import Tempus.Guards, only: [is_locator: 1, is_coming_before: 2]\n    ...> is_locator(Date.utc_today())\n    true\n    ...> is_locator(& Date.utc_today() |> Tempus.Slot.wrap() |> is_coming_before(&1))\n    true\n    ...> is_locator(true)\n    false","ref":"Tempus.Guards.html#is_locator/1-examples"},{"type":"macro","title":"Tempus.Guards.is_origin/1","doc":"Guard to validate that the term given is actually a `t:Tempus.Slot.origin/0`","ref":"Tempus.Guards.html#is_origin/1"},{"type":"macro","title":"Examples - Tempus.Guards.is_origin/1","doc":"iex> import Tempus.Guards, only: [is_origin: 1]\n    ...> is_origin(Date.utc_today())\n    true\n    ...> is_origin(nil)\n    true\n    ...> is_origin(:ok)\n    false","ref":"Tempus.Guards.html#is_origin/1-examples"},{"type":"macro","title":"Tempus.Guards.is_slot/1","doc":"Syntactic sugar for `is_struct(term, Tempus.Slot)`.","ref":"Tempus.Guards.html#is_slot/1"},{"type":"macro","title":"Tempus.Guards.is_slot_border/2","doc":"Guard to validate whether the `t:DateTime.t/0` given as the first argument\n  is the border of the slot.","ref":"Tempus.Guards.html#is_slot_border/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_slot_border/2","doc":"iex> import Tempus.Guards, only: [is_slot_border: 2]\n    iex> dt = DateTime.utc_now()\n    ...> is_slot_border(dt, %Tempus.Slot{from: dt, to: nil})\n    true\n    iex> is_slot_border(dt, Tempus.Slot.wrap(Date.utc_today()))\n    false","ref":"Tempus.Guards.html#is_slot_border/2-examples"},{"type":"macro","title":"Tempus.Guards.is_slot_coming_before/2","doc":"Guard to validate if two `Tempus.Slot` instances passed as parameters if the former one\n  is less (comes earlier) than the latter one.\n\nIf arguments have different timezones, returns a meaningful value for positive unix epoch only.\n\nIf arguments are not slots, or timezones differ and values are before epoch, returns `false`.","ref":"Tempus.Guards.html#is_slot_coming_before/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_slot_coming_before/2","doc":"iex> import Tempus.Guards, only: [is_slot_coming_before: 2]\n    ...> slot = Tempus.Slot.wrap(~U|2020-01-01T12:00:00Z|)\n    ...> is_slot_coming_before(slot, Tempus.Slot.shift_tz(slot, \"Australia/Sydney\"))\n    false\n    ...> is_slot_coming_before(slot, Tempus.Slot.wrap(~U|2020-01-01T23:59:59Z|))\n    true","ref":"Tempus.Guards.html#is_slot_coming_before/2-examples"},{"type":"macro","title":"Tempus.Guards.is_slot_covered/2","doc":"Guard to validate if the slot instance passed as a first argument is covered by the slot passed last.","ref":"Tempus.Guards.html#is_slot_covered/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_slot_covered/2","doc":"iex> import Tempus.Guards, only: [is_slot_covered: 2]\n    ...> slot = Tempus.slot!(~U|2020-01-01T12:00:00Z|, ~U|2020-01-01T23:59:59Z|)\n    ...> covering = Tempus.Slot.wrap(~D|2020-01-01|)\n    ...> joint = Tempus.slot!(~U|2020-01-01T06:00:00Z|, ~U|2020-01-01T18:00:00Z|)\n    ...> is_slot_covered(slot, covering)\n    true\n    iex> is_slot_covered(slot, joint)\n    false","ref":"Tempus.Guards.html#is_slot_covered/2-examples"},{"type":"macro","title":"Tempus.Guards.is_slot_equal/2","doc":"Guard to validate whether two slots are equal","ref":"Tempus.Guards.html#is_slot_equal/2"},{"type":"macro","title":"Examples - Tempus.Guards.is_slot_equal/2","doc":"iex> import Tempus.Guards, only: [is_slot_equal: 2]\n    ...> import Tempus.Sigils\n    ...> s1 = ~I[2023-04-09 00:00:00Z|2023-04-09 23:59:59.999999Z]\n    ...> s2 = Tempus.Slot.wrap(~D|2023-04-09|)\n    ...> s3 = ~I[2023-04-09 00:00:00Z|2023-04-09 23:59:59Z]\n    ...> is_slot_equal(s1, s1)\n    true\n    iex> is_slot_equal(s1, s2)\n    true\n    iex> is_slot_equal(s1, s3)\n    false\n    iex> s_bcn = ~U[2023-06-26T09:30:00Z]\n    ...> {:ok, s_ny} = DateTime.shift_zone(s_bcn, \"America/New_York\")\n    ...> is_slot_equal(Tempus.Slot.wrap(s_bcn), Tempus.Slot.wrap(s_ny))\n    true","ref":"Tempus.Guards.html#is_slot_equal/2-examples"},{"type":"macro","title":"Tempus.Guards.is_slot_nil/1","doc":"Guard to validate whether the slot is `nil` (has neither end set.)","ref":"Tempus.Guards.html#is_slot_nil/1"},{"type":"macro","title":"Examples - Tempus.Guards.is_slot_nil/1","doc":"iex> import Tempus.Guards, only: [is_slot_nil: 1]\n    iex> is_slot_nil(Tempus.Slot.id())\n    true\n    iex> is_slot_nil(Tempus.Slot.wrap(Date.utc_today()))\n    false\n    iex> is_slot_nil(:ok)\n    false","ref":"Tempus.Guards.html#is_slot_nil/1-examples"},{"type":"macro","title":"Tempus.Guards.is_slot_open/1","doc":"Guard to validate whether the slot is open (has either end not set)\n\nPlease note, that the slot having both ends set to `nil` is considered\n  a special case and is not reported as _open_.","ref":"Tempus.Guards.html#is_slot_open/1"},{"type":"macro","title":"Examples - Tempus.Guards.is_slot_open/1","doc":"iex> import Tempus.Guards, only: [is_slot_open: 1]\n    iex> is_slot_open(%Tempus.Slot{from: nil, to: DateTime.utc_now()})\n    true\n    iex> is_slot_open(Tempus.Slot.wrap(Date.utc_today()))\n    false\n    iex> is_slot_open(:ok)\n    false","ref":"Tempus.Guards.html#is_slot_open/1-examples"},{"type":"macro","title":"Tempus.Guards.is_time/1","doc":"Syntactic sugar for `is_struct(term, Time)`.","ref":"Tempus.Guards.html#is_time/1"},{"type":"function","title":"Tempus.Guards.joint_in_delta?/3","doc":"Helper to validate one slot overlaps another in delta. Unlike guards,\n  this function does not expect arguments in the correct order, and would return\n  `true` if the slots overlap even if `s2` comes _before_ `s1`.","ref":"Tempus.Guards.html#joint_in_delta?/3"},{"type":"function","title":"Examples - Tempus.Guards.joint_in_delta?/3","doc":"iex> import Tempus.Guards, only: [joint_in_delta?: 3]\n    ...> import Tempus.Sigils\n    ...> s1 = ~I[2023-04-09 23:00:00Z|2023-04-09 23:59:59Z]\n    ...> joint_in_delta?(s1, s1, 1)\n    true\n    iex> s2 = Tempus.Slot.wrap(~D|2023-04-10|)\n    ...> joint_in_delta?(s1, s2, 1)\n    true\n    iex> joint_in_delta?(s2, s1, 1)\n    true\n    iex> joint_in_delta?(s1, s2, microsecond: 500)\n    false","ref":"Tempus.Guards.html#joint_in_delta?/3-examples"},{"type":"macro","title":"Tempus.Guards.to_unix/2","doc":"Macro to convert the `DateTime` struct to epoch. Unlike `DateTime.to_unix/2`,\n  it does not support different `Calendar`s but `Calendar.ISO`, and it does not\n  support negative epoch times.\n\nIt although supports timezones and can be used in guards.","ref":"Tempus.Guards.html#to_unix/2"},{"type":"macro","title":"Examples - Tempus.Guards.to_unix/2","doc":"iex> import Tempus.Guards, only: [to_unix: 2]\n    ...> dt = DateTime.utc_now()\n    ...> to_unix(dt, :second) == DateTime.to_unix(dt, :second)\n    true\n    ...> to_unix(dt, :millisecond) == DateTime.to_unix(dt, :millisecond)\n    true\n    ...> to_unix(dt, :microsecond) == DateTime.to_unix(dt, :microsecond)\n    true\n    ...> to_unix(~U[2024-01-16 14:50:59.787204Z], :microsecond)\n    1705416659787204\n\nAllowed in guard tests. Inlined by the compiler.","ref":"Tempus.Guards.html#to_unix/2-examples"},{"type":"module","title":"Tempus.Sigils","doc":"Handy sigils to instantiate `Tempus.Slot` and `Tempus.Slots`","ref":"Tempus.Sigils.html"},{"type":"function","title":"Tempus.Sigils.parse/1","doc":"Parses the sigil-like binary representation of a `Tempus.Slot`.","ref":"Tempus.Sigils.html#parse/1"},{"type":"function","title":"Examples - Tempus.Sigils.parse/1","doc":"iex> Tempus.Sigils.parse(\"2021-03-30T06:35:40Z|2021-03-30T06:36:00Z\")\n    {:ok, %Tempus.Slot{from: ~U[2021-03-30 06:35:40Z], to: ~U[2021-03-30 06:36:00Z]}}\n    iex> Tempus.Sigils.parse(\"foo|bar\")\n    {:error, :invalid_format}","ref":"Tempus.Sigils.html#parse/1-examples"},{"type":"macro","title":"Tempus.Sigils.sigil_I/2","doc":"Handles the sigil `~I` for `Tempus.Slot`.\nIt returns a slot without interpolations and without escape\ncharacters, except for the escaping of the closing sigil character\nitself.\n\n_Allowed separators:_ `|`, `→`, `->`, `..` (might be surrounded by spaces)\n\n_Allowed modifiers:_\n\n- _none_ — expects two instances of `DateTime`\n- _single letter_, one of `d`, `t`, `u` — both values are expected to be of that type\n- _any combination_ of two letters `d`, `t`, `u` — the values are treated respectively\n- `g` — the value types are to be guessed","ref":"Tempus.Sigils.html#sigil_I/2"},{"type":"macro","title":"Examples - Tempus.Sigils.sigil_I/2","doc":"iex> import Tempus.Sigils\n    iex> ~I(2021-03-30T06:35:40Z|2021-03-30T06:36:00Z)\n    %Tempus.Slot{from: ~U[2021-03-30 06:35:40Z], to: ~U[2021-03-30 06:36:00Z]}\n    iex> ~I(2021-03-30 → 2021-03-31)d\n    %Tempus.Slot{from: ~U[2021-03-30 00:00:00.000000Z], to: ~U[2021-03-31 23:59:59.999999Z]}\n    iex> ~I(2021-03-30)d\n    %Tempus.Slot{from: ~U[2021-03-30 00:00:00.000000Z], to: ~U[2021-03-30 23:59:59.999999Z]}\n    iex> ~I(2021-03-30 06:35:40Z .. 2021-03-30 06:36:00Z)g\n    %Tempus.Slot{from: ~U[2021-03-30 06:35:40Z], to: ~U[2021-03-30 06:36:00Z]}\n    iex> ~I(|2021-03-30 06:36:00Z)g\n    %Tempus.Slot{from: ~U[2021-03-30 06:36:00Z], to: ~U[2021-03-30 06:36:00Z]}\n    iex> ~I(2021-03-30 06:36:00Z|)g\n    %Tempus.Slot{from: ~U[2021-03-30 06:36:00Z], to: ~U[2021-03-30 06:36:00Z]}\n    iex> Code.compile_quoted(quote do: ~I(foo|bar)g)\n    ** (CompileError) nofile: `~I` sigil input is malformed, error: :invalid_format","ref":"Tempus.Sigils.html#sigil_I/2-examples"},{"type":"module","title":"Tempus.Slot","doc":"Declares a timeslot and exports functions to check whether the given date\n  and/or datetime is covered by this slot or not.\n\nThis module probably should not be called directly.","ref":"Tempus.Slot.html"},{"type":"function","title":"Tempus.Slot.compare/3","doc":"Compares two slot structs.\n\nReturns `:gt` if first slot is strictly later than the second and `:lt` for vice versa.\n**NB** `:eq` is returned not only if slots are equal, but also when they are overlapped.\n\nMight be used in `Enum.sort/2`.","ref":"Tempus.Slot.html#compare/3"},{"type":"function","title":"Examples - Tempus.Slot.compare/3","doc":"iex> slot = %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-10-02 23:59:59.999999Z]}\n    iex> slot1 = %Tempus.Slot{from: nil, to: ~U[2020-09-30 00:00:00.000000Z]}\n    iex> slot2 = %Tempus.Slot{from: nil, to: DateTime.utc_now()}\n    iex> slot3 = %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: nil}\n    iex> slot4 = %Tempus.Slot{from: DateTime.utc_now(), to: nil}\n    iex> Tempus.Slot.compare(Tempus.Slot.id(), Tempus.Slot.id(), true)\n    :eq\n    iex> Tempus.Slot.compare(slot1, slot2, false)\n    :eq\n    iex> Tempus.Slot.compare(slot1, slot2, true)\n    :joint\n    iex> Tempus.Slot.compare(slot3, slot4, false)\n    :eq\n    iex> Tempus.Slot.compare(slot3, slot4, true)\n    :joint\n    iex> Tempus.Slot.compare(slot, slot, true)\n    :eq\n    iex> Tempus.Slot.compare(slot, DateTime.utc_now(), true)\n    :lt\n    iex> Tempus.Slot.compare(slot, ~D|2000-01-01|, true)\n    :gt\n    iex> Tempus.Slot.compare(slot, slot.from, true)\n    :joint","ref":"Tempus.Slot.html#compare/3-examples"},{"type":"function","title":"Tempus.Slot.cover?/3","doc":"Checks whether to `Slot` covers the data/datetime passed as a second argument.","ref":"Tempus.Slot.html#cover?/3"},{"type":"function","title":"Examples - Tempus.Slot.cover?/3","doc":"iex> dt_between = ~U|2015-09-30 01:00:00Z|\n    ...> dt_from = ~U|2015-09-30 00:00:00Z|\n    ...> dt_to = ~U|2015-10-01 01:00:00Z|\n    ...> d_from = Date.from_iso8601!(\"2015-09-30\")\n    ...> d_to = Date.from_iso8601!(\"2015-10-01\")\n    iex> slot = %Tempus.Slot{from: dt_from, to: dt_to}\n    iex> Tempus.Slot.cover?(slot, dt_between)\n    true\n    iex> Tempus.Slot.cover?(slot, dt_to)\n    true\n    iex> Tempus.Slot.cover?(slot, dt_to, true)\n    false\n    iex> Tempus.Slot.cover?(slot, d_from)\n    true\n    iex> Tempus.Slot.cover?(slot, d_from, true)\n    false\n    iex> Tempus.Slot.cover?(slot, ~U|2000-01-01 00:00:00Z|)\n    false\n    iex> Tempus.Slot.cover?(slot, d_to)\n    false","ref":"Tempus.Slot.html#cover?/3-examples"},{"type":"function","title":"Tempus.Slot.disjoint?/2","doc":"Returns `true` if two slots are disjoined, `false` otherwise.","ref":"Tempus.Slot.html#disjoint?/2"},{"type":"function","title":"Examples - Tempus.Slot.disjoint?/2","doc":"iex> slot = %Tempus.Slot{from: ~U|2015-09-01 00:00:00Z|, to: ~U|2015-10-01 00:00:00Z|}\n    iex> inner = %Tempus.Slot{from: ~U|2015-09-01 00:00:00Z|, to: ~U|2015-09-01 01:00:00Z|}\n    iex> Tempus.Slot.disjoint?(slot, inner)\n    false\n    iex> inner = %Tempus.Slot{from: ~U|2015-09-01 00:00:00Z|, to: ~U|2015-10-01 01:00:00Z|}\n    iex> Tempus.Slot.disjoint?(slot, inner)\n    false\n    iex> outer = %Tempus.Slot{from: ~U|2015-10-01 00:00:01Z|, to: ~U|2015-10-01 01:00:00Z|}\n    iex> Tempus.Slot.disjoint?(slot, outer)\n    true\n    iex> Tempus.Slot.disjoint?(~D|2000-01-01|, ~U|2015-10-01 00:00:01Z|)\n    true","ref":"Tempus.Slot.html#disjoint?/2-examples"},{"type":"function","title":"Tempus.Slot.duration/2","doc":"Calculates the duration of a slot in units given as a second parameter\n  (default: `:second`.)","ref":"Tempus.Slot.html#duration/2"},{"type":"function","title":"Example - Tempus.Slot.duration/2","doc":"iex> Tempus.Slot.duration(~D|2020-09-03|)\n    86400\n    iex> Tempus.Slot.duration(Tempus.Slot.id())\n    0\n    iex> Tempus.Slot.duration(%Tempus.Slot{from: nil, to: DateTime.utc_now()})\n    :infinity\n    iex> Tempus.Slot.duration(%Tempus.Slot{from: DateTime.utc_now(), to: nil})\n    :infinity","ref":"Tempus.Slot.html#duration/2-example"},{"type":"function","title":"Tempus.Slot.id/0","doc":"Identity element, void slot `~I[nil → nil]`","ref":"Tempus.Slot.html#id/0"},{"type":"function","title":"Tempus.Slot.intersect/1","doc":"Intersects slots to the minimal covered timeslice.","ref":"Tempus.Slot.html#intersect/1"},{"type":"function","title":"Example - Tempus.Slot.intersect/1","doc":"iex> Tempus.Slot.intersect([Tempus.Slot.id(), Tempus.Slot.id()])\n    Tempus.Slot.id()\n\n    iex> Tempus.Slot.intersect([%Tempus.Slot{from: nil, to: ~U[2020-09-30 23:00:00Z]},\n    ...>   %Tempus.Slot{from: nil, to: ~U[2020-09-30 23:00:00Z]}])\n    %Tempus.Slot{from: nil, to: ~U[2020-09-30 23:00:00Z]}\n\n    iex> Tempus.Slot.intersect([%Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: nil},\n    ...>   %Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: nil}])\n    %Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: nil}\n\n    iex> Tempus.Slot.intersect([~D|2020-09-30|, Tempus.Slot.id()])\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-09-30 23:59:59.999999Z]}\n\n    iex> Tempus.Slot.intersect([Tempus.Slot.id(), ~D|2020-09-30|])\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-09-30 23:59:59.999999Z]}\n\n    iex> Tempus.Slot.intersect([~D|2020-09-30|,\n    ...>   %Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: nil}])\n    %Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: ~U[2020-09-30 23:59:59.999999Z]}\n\n    iex> Tempus.Slot.intersect([~D|2020-09-30|,\n    ...>   %Tempus.Slot{from: nil, to: ~U[2020-09-30 23:00:00Z]}])\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-09-30 23:00:00Z]}\n\n    iex> Tempus.Slot.intersect([\n    ...>   %Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: nil}, ~D|2020-09-30|])\n    %Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: ~U[2020-09-30 23:59:59.999999Z]}\n\n    iex> Tempus.Slot.intersect([\n    ...>   %Tempus.Slot{from: nil, to: ~U[2020-09-30 23:00:00Z]}, ~D|2020-09-30|])\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-09-30 23:00:00Z]}\n\n    iex> Tempus.Slot.intersect([Tempus.Slot.wrap(~D|2020-09-30|),\n    ...>   %Tempus.Slot{from: ~U|2020-09-30 23:00:00Z|, to: ~U|2020-10-02 00:00:00Z|}])\n    %Tempus.Slot{from: ~U[2020-09-30 23:00:00Z], to: ~U[2020-09-30 23:59:59.999999Z]}\n\n    iex> Tempus.Slot.intersect([~D|2020-09-30|, ~D|2000-09-30|,\n    ...>   %Tempus.Slot{from: ~U|2020-09-30 23:00:00Z|, to: ~U|2020-10-02 00:00:00Z|}])\n    nil","ref":"Tempus.Slot.html#intersect/1-example"},{"type":"function","title":"Tempus.Slot.join/1","doc":"Joins slots to the maximal covered timeslice.","ref":"Tempus.Slot.html#join/1"},{"type":"function","title":"Example - Tempus.Slot.join/1","doc":"iex> Tempus.Slot.join([])\n    Tempus.Slot.id()\n\n    iex> Tempus.Slot.join([Tempus.Slot.wrap(~D|2020-09-30|), Tempus.Slot.wrap(~D|2020-10-02|)])\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-10-02 23:59:59.999999Z]}\n\n    iex> Tempus.Slot.join([~D|2020-09-30|, ~D|2020-10-02|])\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-10-02 23:59:59.999999Z]}","ref":"Tempus.Slot.html#join/1-example"},{"type":"function","title":"Tempus.Slot.join/2","doc":"Joins two slots to the maximal covered timeslice.","ref":"Tempus.Slot.html#join/2"},{"type":"function","title":"Example - Tempus.Slot.join/2","doc":"iex> Tempus.Slot.join(Tempus.Slot.wrap(~D|2020-09-30|), Tempus.Slot.wrap(~D|2020-10-02|))\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-10-02 23:59:59.999999Z]}\n\n    iex> Tempus.Slot.join(~D|2020-09-30|, ~D|2020-10-02|)\n    %Tempus.Slot{from: ~U[2020-09-30 00:00:00.000000Z], to: ~U[2020-10-02 23:59:59.999999Z]}","ref":"Tempus.Slot.html#join/2-example"},{"type":"function","title":"Tempus.Slot.neighbour?/2","doc":"Returns `true` if two slots are neighbours, `false` otherwise.","ref":"Tempus.Slot.html#neighbour?/2"},{"type":"function","title":"Examples - Tempus.Slot.neighbour?/2","doc":"iex> slot = %Tempus.Slot{from: ~U|2015-09-01 00:00:00Z|, to: ~U|2015-10-01 23:59:59Z|}\n    iex> Tempus.Slot.neighbour?(slot, Tempus.Slot.wrap(~D|2015-10-02|))\n    true\n    iex> Tempus.Slot.neighbour?(slot, Tempus.Slot.wrap(~D|2015-08-31|))\n    true\n    iex> Tempus.Slot.neighbour?(slot, Tempus.Slot.wrap(~D|2015-10-01|))\n    false\n    iex> Tempus.Slot.neighbour?(slot, Tempus.Slot.wrap(~D|2015-10-03|))\n    false","ref":"Tempus.Slot.html#neighbour?/2-examples"},{"type":"function","title":"Tempus.Slot.new/1","doc":"Creates new slot using `arg[:from]` as a starting origin and `arg[:to]` and an ending origin.","ref":"Tempus.Slot.html#new/1"},{"type":"function","title":"Examples - Tempus.Slot.new/1","doc":"iex> Tempus.Slot.new(from: ~U|2015-09-30 00:00:00Z|, to: ~U|2015-10-01 01:00:00Z|)\n    {:ok, %Tempus.Slot{from: ~U|2015-09-30 00:00:00Z|, to: ~U|2015-10-01 01:00:00Z|}}\n    iex> Tempus.Slot.new(%{from: ~D|2015-09-30|, to: ~U|2015-10-01T12:00:00Z|})\n    {:ok, %Tempus.Slot{from: ~U|2015-09-30 00:00:00.000000Z|, to: ~U|2015-10-01 12:00:00Z|}}","ref":"Tempus.Slot.html#new/1-examples"},{"type":"function","title":"Tempus.Slot.new/2","doc":"Creates new slot using `from` as a starting origin and `to` and an ending origin.\nSee `new/1` for more readable implementation.","ref":"Tempus.Slot.html#new/2"},{"type":"function","title":"Examples - Tempus.Slot.new/2","doc":"iex> import Tempus.Sigils\n    iex> Tempus.Slot.new(~U|2015-09-30 00:00:00Z|, ~U|2015-10-01 01:00:00Z|)\n    {:ok, %Tempus.Slot{from: ~U|2015-09-30 00:00:00Z|, to: ~U|2015-10-01 01:00:00Z|}}\n    iex> Tempus.Slot.new(~D|2015-09-30|, ~U|2015-10-01T12:00:00Z|)\n    {:ok, %Tempus.Slot{from: ~U|2015-09-30 00:00:00.000000Z|, to: ~U|2015-10-01 12:00:00Z|}}\n    iex> Tempus.Slot.new(nil, nil)\n    {:ok, Tempus.Slot.id()}\n    iex> Tempus.Slot.new(~D|2015-09-30|, nil)\n    {:ok, ~I(2015-09-30T00:00:00.000000Z → ∞)un}\n    iex> Tempus.Slot.new(nil, ~D|2015-09-30|)\n    {:ok, ~I(∞ → 2015-09-30T23:59:59.999999Z)nu}\n    iex> Tempus.Slot.new(:ok, :ok)\n    {:error, :invalid_input}","ref":"Tempus.Slot.html#new/2-examples"},{"type":"function","title":"Tempus.Slot.new!/2","doc":"Creates new slot using `from` as a starting origin and `to` and an ending origin.\nUnlike `new/1`, this function raises on malformed input.","ref":"Tempus.Slot.html#new!/2"},{"type":"function","title":"Examples - Tempus.Slot.new!/2","doc":"iex> Tempus.Slot.new!(~U|2015-09-30 00:00:00Z|, ~U|2015-10-01 01:00:00Z|)\n    %Tempus.Slot{from: ~U|2015-09-30 00:00:00Z|, to: ~U|2015-10-01 01:00:00Z|}\n    iex> Tempus.Slot.new!(~D|2015-09-30|, ~U|2015-10-01T12:00:00Z|)\n    %Tempus.Slot{from: ~U|2015-09-30 00:00:00.000000Z|, to: ~U|2015-10-01 12:00:00Z|}\n    iex> Tempus.Slot.new!(:ok, :ok)\n    ** (ArgumentError) malformed from/to argument, expected `origin`","ref":"Tempus.Slot.html#new!/2-examples"},{"type":"function","title":"Tempus.Slot.shift_tz/3","doc":"Shifts both `from` and `to` values to `UTC` zone.","ref":"Tempus.Slot.html#shift_tz/3"},{"type":"function","title":"Examples - Tempus.Slot.shift_tz/3","doc":"```elixir\nslot = %Tempus.Slot{\n   from: DateTime.from_naive!(~N|2018-01-05 21:00:00|, \"America/New_York\"),\n   to: DateTime.from_naive!(~N|2018-01-08 08:59:59|, \"Australia/Sydney\")\n}\n#⇒ %Tempus.Slot{from: ~U[2018-01-06 02:00:00Z], to: ~U[2018-01-07 21:59:59Z]}\n```","ref":"Tempus.Slot.html#shift_tz/3-examples"},{"type":"function","title":"Tempus.Slot.strict_compare/2","doc":"Compares two slot structs. The same as `compare/2`, but returns `:joint` if\nthe slots are overlapped.","ref":"Tempus.Slot.html#strict_compare/2"},{"type":"function","title":"Examples - Tempus.Slot.strict_compare/2","doc":"iex> Tempus.Slot.strict_compare(~D|2020-01-01|, DateTime.utc_now())\n    :lt","ref":"Tempus.Slot.html#strict_compare/2-examples"},{"type":"function","title":"Tempus.Slot.valid?/1","doc":"Checks whether the `Slot` is valid (to > from) or not.","ref":"Tempus.Slot.html#valid?/1"},{"type":"function","title":"Examples - Tempus.Slot.valid?/1","doc":"iex> slot = %Tempus.Slot{from: ~U|2015-09-30 00:00:00Z|, to: ~U|2015-10-01 01:00:00Z|}\n    iex> Tempus.Slot.valid?(slot)\n    true\n    iex> Tempus.Slot.valid?(%Tempus.Slot{from: slot.to, to: slot.from})\n    false\n    iex> slot = %Tempus.Slot{from: nil, to: ~U|2015-10-01 01:00:00Z|}\n    ...> Tempus.Slot.valid?(slot)\n    true\n    iex> slot = %Tempus.Slot{from: ~U|2015-09-30 00:00:00Z|, to: nil}\n    ...> Tempus.Slot.valid?(slot)\n    true\n    iex> Tempus.Slot.valid?(:ok)\n    false","ref":"Tempus.Slot.html#valid?/1-examples"},{"type":"macro","title":"Tempus.Slot.void/0","doc":"Helper macro to pattern-match void slots.","ref":"Tempus.Slot.html#void/0"},{"type":"function","title":"Tempus.Slot.wrap/2","doc":"Wraps the argument into a slot. For `DateTime` it’d be a single microsecond.\nFor a `Date`, it would be the whole day, starting at `00:00:00.000000` and\n    ending at `23:59:59:999999`.","ref":"Tempus.Slot.html#wrap/2"},{"type":"function","title":"Examples - Tempus.Slot.wrap/2","doc":"iex> Tempus.Slot.wrap(~D|2020-08-06|)\n    %Tempus.Slot{from: ~U[2020-08-06 00:00:00.000000Z], to: ~U[2020-08-06 23:59:59.999999Z]}\n    iex> Tempus.Slot.wrap(:ok)\n    Tempus.Slot.id()","ref":"Tempus.Slot.html#wrap/2-examples"},{"type":"function","title":"Tempus.Slot.xor/2","doc":"Splits the slot given asa first argument to two on borders given as a second slot.","ref":"Tempus.Slot.html#xor/2"},{"type":"function","title":"Examples - Tempus.Slot.xor/2","doc":"iex> outer = Tempus.Slot.wrap(~D[2023-04-12])\n    ...> {:ok, inner} = Tempus.Slot.new(~U[2023-04-12 12:00:00Z], ~U[2023-04-12 13:00:00Z])\n    iex> Tempus.Slot.xor(outer, inner)\n    [%Tempus.Slot{from: ~U[2023-04-12 00:00:00.000000Z], to: ~U[2023-04-12 12:00:00Z]},\n     %Tempus.Slot{from: ~U[2023-04-12 13:00:00Z], to: ~U[2023-04-12 23:59:59.999999Z]}]\n    iex> Tempus.Slot.xor(outer, inner) == Tempus.Slot.xor(inner, outer)\n    true\n    iex> {:ok, past} = Tempus.Slot.new(~U[2020-04-12 12:00:00Z], ~U[2020-04-12 13:00:00Z])\n    ...> Tempus.Slot.xor(past, inner)\n    [past, inner]\n    ...> Tempus.Slot.xor(inner, past)\n    [past, inner]\n    iex> {:ok, border} = Tempus.Slot.new(~U[2023-04-12 11:00:00Z], ~U[2023-04-12 12:00:00Z])\n    ...> Tempus.Slot.xor(border, inner)\n    [Tempus.Slot.new!(~U[2023-04-12 11:00:00Z], ~U[2023-04-12 13:00:00Z])]","ref":"Tempus.Slot.html#xor/2-examples"},{"type":"type","title":"Tempus.Slot.origin/0","doc":"The origin used in comparisons and calculations","ref":"Tempus.Slot.html#t:origin/0"},{"type":"type","title":"Tempus.Slot.t/0","doc":"A timeslot to be used in `Tempus`","ref":"Tempus.Slot.html#t:t/0"},{"type":"module","title":"Tempus.Slots","doc":"The interface to deal with the implementation of `Tempus.Slot` ordered collection.\n\nComes with several default implementations, backed up by ordered list of slots,\n  by `Stream`, by `AVLTree`¹, and by Year-Month² tree.","ref":"Tempus.Slots.html"},{"type":"module","title":"Examples - Tempus.Slots","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   %Tempus.Slot{\n    ...>       from: ~U|2020-08-07 01:00:00Z|, to: ~U|2020-08-08 01:00:00Z|}]\n    ...> Enum.into(slots, %Tempus.Slots{})\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 01:00:00Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}]}}\n    iex> Enum.map(slots, & &1.from)\n    [~U[2020-08-07 00:00:00.000000Z], ~U[2020-08-10 00:00:00.000000Z], ~U[2020-08-07 01:00:00Z]]","ref":"Tempus.Slots.html#module-examples"},{"type":"function","title":"Tempus.Slots.add/3","doc":"Adds another slot to the slots collection.\n\nJoins slots intersecting with the new one, if any.","ref":"Tempus.Slots.html#add/3"},{"type":"function","title":"Example - Tempus.Slots.add/3","doc":"iex> Tempus.Slots.add(%Tempus.Slots{}, Tempus.Slot.wrap(~D|2020-08-07|))\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}]}}\n\n    iex> %Tempus.Slots{}\n    ...> |> Tempus.Slots.add(Tempus.Slot.wrap(~D|2020-08-07|))\n    ...> |> Tempus.Slots.add(Tempus.Slot.wrap(~D|2020-08-10|))\n    ...> |> Tempus.Slots.add(%Tempus.Slot{\n    ...>       from: ~U|2020-08-07 01:00:00Z|, to: ~U|2020-08-08 01:00:00Z|})\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 01:00:00Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}]}}","ref":"Tempus.Slots.html#add/3-example"},{"type":"function","title":"Tempus.Slots.count/1","doc":"Returns the number of slots","ref":"Tempus.Slots.html#count/1"},{"type":"function","title":"Examples - Tempus.Slots.count/1","doc":"iex> import Tempus.Sigils\n    iex> slots = [\n    ...>   ~I(2000-01-01T00:00:00.000000Z → 2000-01-01T23:59:59.999999Z),\n    ...>   ~I(2010-01-01T00:00:00.000000Z → 2010-01-01T23:59:59.999999Z),\n    ...>   ~I(2020-01-01T00:00:00.000000Z → 2020-01-01T23:59:59.999999Z)\n    ...> ]\n    iex> slots |> Tempus.Slots.new() |> Tempus.Slots.count()\n    3","ref":"Tempus.Slots.html#count/1-examples"},{"type":"function","title":"Tempus.Slots.drop_until/3","doc":"","ref":"Tempus.Slots.html#drop_until/3"},{"type":"function","title":"Tempus.Slots.inverse/2","doc":"Inverses `Slots` returning the new `Slots` instance with slots set where\n  there were blanks.","ref":"Tempus.Slots.html#inverse/2"},{"type":"function","title":"Example - Tempus.Slots.inverse/2","doc":"iex> [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-08|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-12|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    ...> |> Tempus.Slots.inverse()\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: nil, to: ~U[2020-08-06 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-09 00:00:00.000000Z], to: ~U[2020-08-09 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-11 00:00:00.000000Z], to: ~U[2020-08-11 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-13 00:00:00.000000Z], to: nil}]}}\n\n    iex> [\n    ...>   %Tempus.Slot{to: ~U[2020-08-08 23:59:59.999999Z]},\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   %Tempus.Slot{from: ~U[2020-08-12 00:00:00.000000Z]}\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    ...> |> Tempus.Slots.inverse()\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-09 00:00:00.000000Z], to: ~U[2020-08-09 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-11 00:00:00.000000Z], to: ~U[2020-08-11 23:59:59.999999Z]}]}}","ref":"Tempus.Slots.html#inverse/2-example"},{"type":"function","title":"Tempus.Slots.merge/3","doc":"Merges many slots into the first element in the list given as an argument.\n\nOther arguments might be enumerables, the first one must be a `Tempus.Slots` instance.","ref":"Tempus.Slots.html#merge/3"},{"type":"function","title":"Examples - Tempus.Slots.merge/3","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots{})\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}]}}\n    iex> other = [\n    ...>   %Tempus.Slot{from: ~U|2020-08-07 23:00:00Z|, to: ~U|2020-08-08 12:00:00Z|},\n    ...>   %Tempus.Slot{from: ~U|2020-08-12 23:00:00Z|, to: ~U|2020-08-12 23:30:00Z|}\n    ...> ]\n    iex> Tempus.Slots.merge([slots, other])\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 12:00:00Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-12 23:00:00Z], to: ~U[2020-08-12 23:30:00Z]}]}}\n    iex> Tempus.Slots.merge([slots, Tempus.Slots.wrap(other, Tempus.Slots.Stream)]) |> Enum.to_list()\n    [%Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 12:00:00Z]},\n     %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]},\n     %Tempus.Slot{from: ~U[2020-08-12 23:00:00Z], to: ~U[2020-08-12 23:30:00Z]}]","ref":"Tempus.Slots.html#merge/3-examples"},{"type":"function","title":"Tempus.Slots.new/2","doc":"Creates an instance of slots, using a backend given as a parameter.","ref":"Tempus.Slots.html#new/2"},{"type":"function","title":"Examples - Tempus.Slots.new/2","doc":"iex> import Tempus.Sigils\n    iex> slots = [\n    ...>   ~I(2000-01-01T00:00:00.000000Z → 2000-01-01T23:59:59.999999Z),\n    ...>   ~I(2010-01-01T00:00:00.000000Z → 2010-01-01T23:59:59.999999Z),\n    ...>   ~I(2020-01-01T00:00:00.000000Z → 2020-01-01T23:59:59.999999Z)\n    ...> ]\n    iex> Tempus.Slots.new([~D|2000-01-01|, ~D|2010-01-01|, ~D|2020-01-01|])\n    %Tempus.Slots{slots: Enum.into(slots, Tempus.Slots.List.new())}","ref":"Tempus.Slots.html#new/2-examples"},{"type":"function","title":"Tempus.Slots.span/3","doc":"","ref":"Tempus.Slots.html#span/3"},{"type":"function","title":"Tempus.Slots.take_until/3","doc":"","ref":"Tempus.Slots.html#take_until/3"},{"type":"function","title":"Tempus.Slots.wrap/2","doc":"Wraps the argument into a slots instance. For `nil` it’d be an empty slots.\nFor everything else it’d call `Slot.wrap/1` on an argument and add it to empty slots.","ref":"Tempus.Slots.html#wrap/2"},{"type":"function","title":"Examples - Tempus.Slots.wrap/2","doc":"iex> Tempus.Slots.wrap(~D|2020-08-06|)\n    %Tempus.Slots{slots: %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-06 00:00:00.000000Z], to: ~U[2020-08-06 23:59:59.999999Z]}]}}","ref":"Tempus.Slots.html#wrap/2-examples"},{"type":"type","title":"Tempus.Slots.container/0","doc":"","ref":"Tempus.Slots.html#t:container/0"},{"type":"type","title":"Tempus.Slots.implementation/1","doc":"","ref":"Tempus.Slots.html#t:implementation/1"},{"type":"type","title":"Tempus.Slots.implementation/2","doc":"","ref":"Tempus.Slots.html#t:implementation/2"},{"type":"type","title":"Tempus.Slots.locator/0","doc":"The type to use in navigation and/or rewinding slots enumerables","ref":"Tempus.Slots.html#t:locator/0"},{"type":"type","title":"Tempus.Slots.t/0","doc":"","ref":"Tempus.Slots.html#t:t/0"},{"type":"type","title":"Tempus.Slots.t/1","doc":"","ref":"Tempus.Slots.html#t:t/1"},{"type":"type","title":"Tempus.Slots.t/2","doc":"","ref":"Tempus.Slots.html#t:t/2"},{"type":"module","title":"Tempus.Slots.List","doc":"The default `List` implementation of `Tempus.Slots` ordered collection.","ref":"Tempus.Slots.List.html"},{"type":"module","title":"Examples - Tempus.Slots.List","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   %Tempus.Slot{\n    ...>       from: ~U|2020-08-07 01:00:00Z|, to: ~U|2020-08-08 01:00:00Z|}]\n    ...> Enum.into(slots, %Tempus.Slots.List{})\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 01:00:00Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}]}\n    iex> Enum.map(slots, & &1.from)\n    [~U[2020-08-07 00:00:00.000000Z], ~U[2020-08-10 00:00:00.000000Z], ~U[2020-08-07 01:00:00Z]]","ref":"Tempus.Slots.List.html#module-examples"},{"type":"function","title":"Tempus.Slots.List.add/3","doc":"Adds another slot to the slots collection implemented as list.\n\nJoins slots intersecting with the new one, if any.","ref":"Tempus.Slots.List.html#add/3"},{"type":"function","title":"Example - Tempus.Slots.List.add/3","doc":"iex> Tempus.Slots.List.add(%Tempus.Slots.List{}, Tempus.Slot.wrap(~D|2020-08-07|))\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}]}\n\n    iex> %Tempus.Slots.List{}\n    ...> |> Tempus.Slots.List.add(Tempus.Slot.wrap(~D|2020-08-07|))\n    ...> |> Tempus.Slots.List.add(Tempus.Slot.wrap(~D|2020-08-10|))\n    ...> |> Tempus.Slots.List.add(%Tempus.Slot{\n    ...>       from: ~U|2020-08-07 01:00:00Z|, to: ~U|2020-08-08 01:00:00Z|})\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 01:00:00Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}]}","ref":"Tempus.Slots.List.html#add/3-example"},{"type":"function","title":"Tempus.Slots.List.inverse/2","doc":"Inverses `Slots` returning the new `Slots` instance with slots set where\n  there were blanks.","ref":"Tempus.Slots.List.html#inverse/2"},{"type":"function","title":"Example - Tempus.Slots.List.inverse/2","doc":"iex> [~D|2020-08-07|, ~D|2020-08-08|, ~D|2020-08-10|, ~D|2020-08-12|]\n    ...> |> Enum.into(%Tempus.Slots.List{})\n    ...> |> Tempus.Slots.List.inverse()\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: nil, to: ~U[2020-08-06 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-09 00:00:00.000000Z], to: ~U[2020-08-09 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-11 00:00:00.000000Z], to: ~U[2020-08-11 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-13 00:00:00.000000Z], to: nil}]}\n\n    iex> [\n    ...>   %Tempus.Slot{to: ~U[2020-08-08 23:59:59.999999Z]},\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   %Tempus.Slot{from: ~U[2020-08-12 00:00:00.000000Z]}\n    ...> ] |> Enum.into(%Tempus.Slots.List{})\n    ...> |> Tempus.Slots.List.inverse()\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-09 00:00:00.000000Z], to: ~U[2020-08-09 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-11 00:00:00.000000Z], to: ~U[2020-08-11 23:59:59.999999Z]}\n    ]}","ref":"Tempus.Slots.List.html#inverse/2-example"},{"type":"function","title":"Tempus.Slots.List.merge/3","doc":"Merges `other` into `this` slots instance. `other` might be `Enum` _or_ `Stream`.\nWhen `other` is a stream, it gets terminated immediately after the last element\nin `this`.","ref":"Tempus.Slots.List.html#merge/3"},{"type":"function","title":"Examples - Tempus.Slots.List.merge/3","doc":"iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(%Tempus.Slots.List{})\n    iex> other = [\n    ...>   %Tempus.Slot{from: ~U|2020-08-07 23:00:00Z|, to: ~U|2020-08-08 12:00:00Z|},\n    ...>   %Tempus.Slot{from: ~U|2020-08-12 23:00:00Z|, to: ~U|2020-08-12 23:30:00Z|}\n    ...> ]\n    iex> Tempus.Slots.merge([%Tempus.Slots{slots: slots}, other]).slots\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 12:00:00Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-12 23:00:00Z], to: ~U[2020-08-12 23:30:00Z]}]}\n    iex> %Tempus.Slots{slots: slots} |> Tempus.Slots.merge(Tempus.Slots.wrap(other, Tempus.Slots.Stream)) |> Enum.to_list()\n    [\n      %Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 12:00:00Z]},\n      %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-12 23:00:00Z], to: ~U[2020-08-12 23:30:00Z]}]","ref":"Tempus.Slots.List.html#merge/3-examples"},{"type":"function","title":"Tempus.Slots.List.new/0","doc":"Creates the new instance of `Tempus.Slots.List` struct. One usually does not need to call this function directly.","ref":"Tempus.Slots.List.html#new/0"},{"type":"function","title":"Tempus.Slots.List.split/3","doc":"Splits the slots by the pivot given as a `Slot.t` or as a function.\n\nTo keep it consistent, the function actually does _split it until_,\nwhich is intuitive when the pivot is given and kinda counter-intuitive\nwhen the _locator_ function is given.\n\nSee the examples below to grasp the reasoning behind this\narchitectural decision.","ref":"Tempus.Slots.List.html#split/3"},{"type":"function","title":"Examples - Tempus.Slots.List.split/3","doc":"iex> import Tempus.Guards\n    ...> import Tempus.Sigils\n    ...> slots =\n    ...> [~D|2020-08-07|, ~D|2020-08-08|, ~D|2020-08-10|, ~D|2020-08-12|]\n    ...> |> Enum.into(%Tempus.Slots.List{})\n    iex> slots |> Tempus.Slots.List.split(~U|2020-08-09T12:00:00Z|)\n    {\n      [~I(2020-08-07T00:00:00.000000Z → 2020-08-07T23:59:59.999999Z), ~I(2020-08-08T00:00:00.000000Z → 2020-08-08T23:59:59.999999Z)],\n      [~I(2020-08-10T00:00:00.000000Z → 2020-08-10T23:59:59.999999Z), ~I(2020-08-12T00:00:00.000000Z → 2020-08-12T23:59:59.999999Z)]\n    }\n    iex> slots |> Tempus.Slots.List.split(&is_slot_coming_before(Tempus.Slot.wrap(~U|2020-08-09T12:00:00Z|), &1))\n    {\n      [~I(2020-08-07T00:00:00.000000Z → 2020-08-07T23:59:59.999999Z), ~I(2020-08-08T00:00:00.000000Z → 2020-08-08T23:59:59.999999Z)],\n      [~I(2020-08-10T00:00:00.000000Z → 2020-08-10T23:59:59.999999Z), ~I(2020-08-12T00:00:00.000000Z → 2020-08-12T23:59:59.999999Z)]\n    }","ref":"Tempus.Slots.List.html#split/3-examples"},{"type":"type","title":"Tempus.Slots.List.t/0","doc":"","ref":"Tempus.Slots.List.html#t:t/0"},{"type":"module","title":"Tempus.Slots.Stream","doc":"The default `Stream` implementation of `Tempus.Slots` ordered collection.","ref":"Tempus.Slots.Stream.html"},{"type":"module","title":"Examples - Tempus.Slots.Stream","doc":"iex> import Tempus.Slots.Stream, only: [slots: 0]\n    iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   %Tempus.Slot{\n    ...>       from: ~U|2020-08-07 01:00:00Z|, to: ~U|2020-08-08 01:00:00Z|}]\n    ...> slots |> Enum.into(slots()) |> Enum.to_list()\n    [%Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 01:00:00Z]},\n     %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}]\n    iex> Enum.map(slots, & &1.from)\n    [~U[2020-08-07 00:00:00.000000Z], ~U[2020-08-10 00:00:00.000000Z], ~U[2020-08-07 01:00:00Z]]","ref":"Tempus.Slots.Stream.html#module-examples"},{"type":"function","title":"Tempus.Slots.Stream.add/3","doc":"Adds another slot to the slots collection backed by stream.\n\nJoins slots intersecting with the new one, if any.","ref":"Tempus.Slots.Stream.html#add/3"},{"type":"function","title":"Example - Tempus.Slots.Stream.add/3","doc":"iex> import Tempus.Slots.Stream, only: [slots: 0]\n    iex> Tempus.Slots.Stream.add(slots(), Tempus.Slot.wrap(~D|2020-08-07|)) |> Enum.to_list()\n    [%Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-07 23:59:59.999999Z]}]\n\n    iex> %Tempus.Slots.Stream{}\n    ...> |> Tempus.Slots.Stream.add(Tempus.Slot.wrap(~D|2020-08-07|))\n    ...> |> Tempus.Slots.Stream.add(Tempus.Slot.wrap(~D|2020-08-10|))\n    ...> |> Tempus.Slots.Stream.add(%Tempus.Slot{\n    ...>       from: ~U|2020-08-07 01:00:00Z|, to: ~U|2020-08-08 01:00:00Z|})\n    ...> |> Enum.to_list()\n    [%Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 01:00:00Z]},\n     %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]}]","ref":"Tempus.Slots.Stream.html#add/3-example"},{"type":"function","title":"Tempus.Slots.Stream.inverse/2","doc":"Inverses `Slots` returning the new `Slots` instance with slots set where\n  there were blanks.","ref":"Tempus.Slots.Stream.html#inverse/2"},{"type":"function","title":"Example - Tempus.Slots.Stream.inverse/2","doc":"iex> [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-08|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-12|)\n    ...> ] |> Enum.into(%Tempus.Slots.List{})\n    ...> |> Tempus.Slots.List.inverse()\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: nil, to: ~U[2020-08-06 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-09 00:00:00.000000Z], to: ~U[2020-08-09 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-11 00:00:00.000000Z], to: ~U[2020-08-11 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-13 00:00:00.000000Z], to: nil}]}\n\n    iex> [\n    ...>   %Tempus.Slot{to: ~U[2020-08-08 23:59:59.999999Z]},\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|),\n    ...>   %Tempus.Slot{from: ~U[2020-08-12 00:00:00.000000Z]}\n    ...> ] |> Enum.into(%Tempus.Slots.List{})\n    ...> |> Tempus.Slots.List.inverse()\n    %Tempus.Slots.List{slots: [\n      %Tempus.Slot{from: ~U[2020-08-09 00:00:00.000000Z], to: ~U[2020-08-09 23:59:59.999999Z]},\n      %Tempus.Slot{from: ~U[2020-08-11 00:00:00.000000Z], to: ~U[2020-08-11 23:59:59.999999Z]}\n    ]}","ref":"Tempus.Slots.Stream.html#inverse/2-example"},{"type":"function","title":"Tempus.Slots.Stream.iterate/3","doc":"Produces a stream of slots wrapped in `Tempus.Slots.Stream`, ensuring the order\n  of elements emitted.","ref":"Tempus.Slots.Stream.html#iterate/3"},{"type":"function","title":"Options - Tempus.Slots.Stream.iterate/3","doc":"- `join` (_default:_ `false`) — `nil | boolean() | non_neg_integer()` specifies how\n  slots are to be joined\n- `return_as` (_default:_ `:stream`) — `:stream | :slots` if slots, the function returns\n  the `t:Tempus.Slots.t/0` instance backed up by produced `t:Temput.Slots.Stream.t/0`\n\nBy default, slots will not be joined. Pass `true` to join if they are 1μsec aside,\n  or an integer specifying the number of μsecs to join.","ref":"Tempus.Slots.Stream.html#iterate/3-options"},{"type":"function","title":"Examples - Tempus.Slots.Stream.iterate/3","doc":"iex> import Tempus.Sigils\n    ...> Tempus.Slots.Stream.iterate(~D|2024-01-20|,\n    ...>   &Tempus.Slot.shift(&1, by: rem(&1.from.day, 2) + 1, unit: :day),\n    ...>   join: true) |> Enum.take(2)\n    [~I(2024-01-20T00:00:00.000000Z → 2024-01-21T23:59:59.999999Z),\n     ~I(2024-01-23T00:00:00.000000Z → 2024-01-23T23:59:59.999999Z)]","ref":"Tempus.Slots.Stream.html#iterate/3-examples"},{"type":"function","title":"Tempus.Slots.Stream.merge/3","doc":"Merges `other` into `this` slots instance. `other` might be `Enum` _or_ `Stream`.\nWhen `other` is a stream, it gets terminated immediately after the last element\nin `this`.","ref":"Tempus.Slots.Stream.html#merge/3"},{"type":"function","title":"Examples - Tempus.Slots.Stream.merge/3","doc":"iex> import Tempus.Slots.Stream, only: [slots: 0]\n    iex> slots = [\n    ...>   Tempus.Slot.wrap(~D|2020-08-07|),\n    ...>   Tempus.Slot.wrap(~D|2020-08-10|)\n    ...> ] |> Enum.into(slots())\n    iex> other = [\n    ...>   %Tempus.Slot{from: ~U|2020-08-07 23:00:00Z|, to: ~U|2020-08-08 12:00:00Z|},\n    ...>   %Tempus.Slot{from: ~U|2020-08-12 23:00:00Z|, to: ~U|2020-08-12 23:30:00Z|}\n    ...> ] |> Enum.into(slots())\n    iex> slots |> Tempus.Slots.Stream.merge(other) |> Enum.to_list()\n    [%Tempus.Slot{from: ~U[2020-08-07 00:00:00.000000Z], to: ~U[2020-08-08 12:00:00Z]},\n     %Tempus.Slot{from: ~U[2020-08-10 00:00:00.000000Z], to: ~U[2020-08-10 23:59:59.999999Z]},\n     %Tempus.Slot{from: ~U[2020-08-12 23:00:00Z], to: ~U[2020-08-12 23:30:00Z]}]","ref":"Tempus.Slots.Stream.html#merge/3-examples"},{"type":"function","title":"Tempus.Slots.Stream.new/0","doc":"Creates the new instance of `Tempus.Slots.Stream` struct. One usually does not need to call this function directly.","ref":"Tempus.Slots.Stream.html#new/0"},{"type":"function","title":"Tempus.Slots.Stream.recurrent/3","doc":"Produces a stream or recurrent weekly slots, given starting and ending `{dow, time, zone}` triples.","ref":"Tempus.Slots.Stream.html#recurrent/3"},{"type":"function","title":"Examples - Tempus.Slots.Stream.recurrent/3","doc":"iex> import Tempus.Sigils\n     ...> Tempus.Slots.Stream.recurrent(Date.from_iso8601!(\"2024-01-22\"), {7, \"15:00:00\", \"Etc/UTC\"}, {3, \"01:00:00\", \"Etc/UTC\"}) |> Enum.take(3)\n     [~I(2024-01-21T15:00:00.000000Z → 2024-01-24T01:00:00.000000Z),\n      ~I(2024-01-28T15:00:00.000000Z → 2024-01-31T01:00:00.000000Z),\n      ~I(2024-02-04T15:00:00.000000Z → 2024-02-07T01:00:00.000000Z)]\n     ...> Tempus.Slots.Stream.recurrent(Date.from_iso8601!(\"2024-01-22\"), {1, \"15:00:00\", \"Etc/UTC\"}, {3, \"01:00:00\", \"Etc/UTC\"}) |> Enum.take(3)\n     [~I(2024-01-22T15:00:00.000000Z → 2024-01-24T01:00:00.000000Z),\n      ~I(2024-01-29T15:00:00.000000Z → 2024-01-31T01:00:00.000000Z),\n      ~I(2024-02-05T15:00:00.000000Z → 2024-02-07T01:00:00.000000Z)]\n     ...> Tempus.Slots.Stream.recurrent(Date.from_iso8601!(\"2024-01-22\"), {1, \"15:00:00\", \"Etc/UTC\"}, {1, \"15:00:00\", \"Australia/Sydney\"}) |> Enum.take(3)\n     [~I(2024-01-15T15:00:00.000000Z → 2024-01-22T15:00:00.000000+11:00),\n      ~I(2024-01-22T15:00:00.000000Z → 2024-01-29T15:00:00.000000+11:00),\n      ~I(2024-01-29T15:00:00.000000Z → 2024-02-05T15:00:00.000000+11:00)]\n     ...> Tempus.Slots.Stream.recurrent(Date.from_iso8601!(\"2024-01-22\"), {1, \"09:00:00\", \"Australia/Sydney\"}, {7, \"23:00:00\", \"Etc/UTC\"}) |> Enum.take(3)\n     [%Tempus.Slot{from: DateTime.new!(~D\"2024-01-29\", ~T\"09:00:00.000000\", \"Australia/Sydney\"), to: ~U|2024-01-28T23:00:00.000000Z|},\n      %Tempus.Slot{from: DateTime.new!(~D\"2024-02-05\", ~T\"09:00:00.000000\", \"Australia/Sydney\"), to: ~U|2024-02-04T23:00:00.000000Z|},\n      %Tempus.Slot{from: DateTime.new!(~D\"2024-02-12\", ~T\"09:00:00.000000\", \"Australia/Sydney\"), to: ~U|2024-02-11T23:00:00.000000Z|}]\n     ...> Tempus.Slots.Stream.recurrent(Date.from_iso8601!(\"2024-03-28\"), {1, \"09:00:00\", \"Australia/Sydney\"}, {7, \"23:00:00\", \"Etc/UTC\"}) |> Enum.take(2)\n     [%Tempus.Slot{from: DateTime.new!(~D\"2024-04-01\", ~T\"09:00:00.000000\", \"Australia/Sydney\"), to: ~U|2024-03-31T23:00:00.000000Z|},\n      %Tempus.Slot{from: DateTime.new!(~D\"2024-04-08\", ~T\"09:00:00.000000\", \"Australia/Sydney\"), to: ~U|2024-04-07T23:00:00.000000Z|}]","ref":"Tempus.Slots.Stream.html#recurrent/3-examples"},{"type":"function","title":"Tempus.Slots.Stream.split/3","doc":"Splits the slots by the pivot given as a `Slot.t` or as a function.\n\nTo keep it consistent, the function actually does _split it until_,\nwhich is intuitive when the pivot is given and kinda counter-intuitive\nwhen the _locator_ function is given.\n\nSee the examples below to grasp the reasoning behind this\narchitectural decision.","ref":"Tempus.Slots.Stream.html#split/3"},{"type":"function","title":"Examples - Tempus.Slots.Stream.split/3","doc":"iex> import Tempus.Guards\n    ...> import Tempus.Sigils\n    iex> slots =\n    ...> [~D|2020-08-07|, ~D|2020-08-08|, ~D|2020-08-10|, ~D|2020-08-12|]\n    ...> |> Enum.into(%Tempus.Slots.Stream{})\n    iex> slots |> Tempus.Slots.Stream.split(~U|2020-08-09T12:00:00Z|) |> Tuple.to_list() |> Enum.map(&Enum.to_list/1)\n    [\n      [~I(2020-08-07T00:00:00.000000Z → 2020-08-07T23:59:59.999999Z), ~I(2020-08-08T00:00:00.000000Z → 2020-08-08T23:59:59.999999Z)],\n      [~I(2020-08-10T00:00:00.000000Z → 2020-08-10T23:59:59.999999Z), ~I(2020-08-12T00:00:00.000000Z → 2020-08-12T23:59:59.999999Z)]\n    ]\n    iex> slots\n    ...> |> Tempus.Slots.Stream.split(&is_slot_coming_before(Tempus.Slot.wrap(~U|2020-08-09T12:00:00Z|), &1))\n    ...> |> Tuple.to_list()\n    ...> |> Enum.map(&Enum.to_list/1)\n    [\n      [~I(2020-08-07T00:00:00.000000Z → 2020-08-07T23:59:59.999999Z), ~I(2020-08-08T00:00:00.000000Z → 2020-08-08T23:59:59.999999Z)],\n      [~I(2020-08-10T00:00:00.000000Z → 2020-08-10T23:59:59.999999Z), ~I(2020-08-12T00:00:00.000000Z → 2020-08-12T23:59:59.999999Z)]\n    ]","ref":"Tempus.Slots.Stream.html#split/3-examples"},{"type":"type","title":"Tempus.Slots.Stream.t/0","doc":"","ref":"Tempus.Slots.Stream.html#t:t/0"},{"type":"exception","title":"Tempus.ArgumentError","doc":"Raised at runtime when an argument passed to function is invalid","ref":"Tempus.ArgumentError.html"},{"type":"extras","title":"Tempus","doc":"# ![Tempus](https://raw.githubusercontent.com/am-kantox/tempus/master/stuff/tempus-48x48.png) Tempus    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/tempus/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/tempus/workflows/Dialyzer/badge.svg)  [![Coverage Status](https://coveralls.io/repos/github/am-kantox/tempus/badge.svg?branch=master)](https://coveralls.io/github/am-kantox/tempus?branch=master)\n\n**Easy handling of time periods aka slots, like business days, holidays, etc.**","ref":"readme.html"},{"type":"extras","title":"Installation - Tempus","doc":"```elixir\ndef deps do\n  [\n    {:tempus, \"~> 0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Changelog - Tempus","doc":"- **`0.15.0`** — [:tada:] `Tempus.parse_cron/2` and low-level `Tempus.Crontab`\n- **`0.14.1`** — [:ant:] `recurrent/3` has now concerned about DST (fuck DST btw)\n- **`0.14.0`** — [:books:] refactoring of guard usage and docs update\n- **`0.13.3`** — [:ant:] avoid crash on the incorrect input of slots\n- **`0.13.2`** — [:tada:] `Tempus.Slots.Stream.recurrent/3` to introduce cron-like streams\n- **`0.13.0`** — [:tada:] different calendars experimental support\n- **`0.12.1`** — [:tada:] timezones and more guards exported\n- **`0.11.0`** — [:books:] better coverage\n- **`0.10.2`** — [:ant:] `add/4` and tests for it: fixed\n- **`0.10.1`** — [:ant:] `split/4` and tests for it: fixed\n- **`0.10.0`** — complete rewrite of implementations, 3–10× faster, `Slots` form an Abelian group now\n- **`0.9.1`** — `Tempus.Sigils.parse/1`\n- **`0.9.0`** — `Tempus.Guards`, `Tempus.slot/{1,2}`, `Tempus.guess/1`, better sigils, prepared for `:tempus_sql`\n- **`0.8.0`** — improve `Tempus.Sigils`, prepared for `:tempus_sql`\n- **`0.7.3`** — `truncate: boolean() | non_neg_integer()` to truncate `Slots` inspection\n- **`0.7.2`** — `Slots.merge/1`\n- **`0.7.1`** — treat infinite slot as special not-covering anything\n- **`0.7.0`** — `merge/2` is 6× faster, `Slot.neighbour?/2`, `Tempus.slice/4`, `Tempus.drop_while/2`, `Tempus.take_while/2`\n- **`0.6.0`** — compatibility with _Elixir v1.14_ (no greek in variables names) meh\n- **`0.5.0`** — stricter `:telemetria` support\n- **`0.4.2`** — make `:telemetria` dependency fully optional\n- **`0.4.0`** — `~I` sigil to ease slots creation, wrap date/times, fancy inspect\n- **`0.3.0`** — drastical performance improvements, benchmarks\n- **`0.2.4`** — fixed bug when finding next free/busy for empty slots\n- **`0.2.3`** — correctly handle empty slots in `next_busy/2`/`next_free/2`\n- **`0.2.2`** — `Slot.shift_tz/3`\n- **`0.2.1`** — accept function as well as stream in `Slots.merge/2`\n- **`0.2.0`** — many improvements, `Tempus.add/2` similar to `DateTime.add/4` but considering slots\n\n## [Documentation](https://hexdocs.pm/tempus)","ref":"readme.html#changelog"},{"type":"extras","title":"Mathematical Background","doc":"# Mathematical Background","ref":"background.html"},{"type":"extras","title":"𝕋erminology - Mathematical Background","doc":"The pair of two elements is called a `Slot` if the following conditions are met\n\n- each element is either `nil` or an instance of a datetime with defined timezone\n- if both elements are datetimes, the first element does not superseed the second one\n- if either element is `nil`, the `Slot` is called _open_, if both are `nil`, it’s called _identity_\n\n`Slot` id denoted `[from → to]`. Let’s define a binary _union_ operation on slots, denoted `∪`. Slots do not form a group with `∪`, but sorted sets of slots of arbitrary length having no joint slots (denoted `𝕥` or more verbose `𝕥[[from₁, to₁], [from₂, to₂], …]`) do indeed form a group, denoted `𝕋`, together with a binary operation `∪` on `𝕋`, such as the following group axioms are satisfied:","ref":"background.html#%F0%9D%95%8Berminology"},{"type":"extras","title":"Associativity - Mathematical Background","doc":"∀ 𝕥₁, 𝕥₂, 𝕥₃ ∈ 𝕋, (𝕥₁ ∪ 𝕥₂) ∪ 𝕥₃ = 𝕥₁ ∪ (𝕥₂ ∪ 𝕥₃)","ref":"background.html#associativity"},{"type":"extras","title":"Identity element - Mathematical Background","doc":"∃ 𝕥₀ ∈ 𝕋 (`𝕥[]`) such that, for every 𝕥 in 𝕋, (𝕥 ∪ 𝕥₀) = (𝕥₀ ∪ 𝕥) = 𝕥","ref":"background.html#identity-element"},{"type":"extras","title":"Inverse element - Mathematical Background","doc":"For each 𝕥 in 𝕋, there exist 𝕥¯¹ such that 𝕥 ∪ 𝕥¯¹ = 𝕥¯¹ ∪ 𝕥 = 𝕥₀\n\nThat said, slots form an Abelian group with union binary operation and empty set as identity element.","ref":"background.html#inverse-element"},{"type":"extras","title":"Slots Semigroup - Mathematical Background","doc":"Slots themselves form a semigroup with a binary union operation, an identity element `[nil → nil]`, without inverse.","ref":"background.html#slots-semigroup"},{"type":"extras","title":"Binary Operation - Mathematical Background","doc":"∀ 𝕥₁ = 𝕥[[from₁ → to₁]], 𝕥₂ = 𝕥[[from₂ → to₂]] ∈ 𝕋, 𝕥₁ ∪ 𝕥₂ is defined as\n\n- 𝕥[[from₁ → to₁], [from₂ → to₂]] if to₁   from₂` or `𝕥[[nil → to₁], [from₂ → to₂]]` otherwise.","ref":"background.html#binary-operation"},{"type":"extras","title":"Mergeability - Mathematical Background","doc":"Once `𝕋` is a group, each two elements of it _might be merged_. Even if they are infinite. That’s why `Stream` implementation of `𝕋` exists.","ref":"background.html#mergeability"},{"type":"extras","title":"Getting Started","doc":"# Getting Started","ref":"getting-started.html"},{"type":"extras","title":"Usecases - Getting Started","doc":"`Tempus` library provides an easy way to handle and manipulate time slots.\n\nTime slot is the base concept behind the library. It is basically the struct, containing `from` and `to` instances of `DateTime`. One might provide the list of slots, as `Enum` or as a `Stream`, and check whether another time slot and/or `DateTime` instance is overlapped or disjoined. Also one might get the next “available” slot(s) providing an `origin`, as well as next busy slot(s).\n\nIt might be used e. g. to calculate the working days between now and some day in the future, providing the list of holidays, and the stream of weekends.","ref":"getting-started.html#usecases"},{"type":"extras","title":"Initializing Slots - Getting Started","doc":"`Tempus.Slots` has an agnostic internal-storage interface for keeping slots. It might have a `List` or `Stream` behind, a self-baked implementation of `Tempus.Group` protocol, ot it might leverage [`AVLTree`](https://github.com/japplegame/avl_tree). It exposes `add/2`, `merge/2` and `inverse/2` functions to add slots, merge the slots with another slot(s) and/or `Stream`, and inverse the slots respecively. Typically one starts with something like\n\n```elixir\nslots = [\n  Tempus.Slot.wrap(~D|2020-08-07|),\n  Tempus.Slot.wrap(~D|2020-08-10|),\n  %Tempus.Slot{\n      from: ~U|2020-08-07 01:00:00Z|, to: ~U|2020-08-08 01:00:00Z|}\n] |> Enum.into(%Tempus.Slots{})\n```\n\nWhen updated, `Tempus.Slots` joins the newly added slot(s) into the set of existing ones, so thet at any moment there is no overlapped slots. This happens when new slot is added with `Tempus.Slots.add/2` and when the enumerable is collected.","ref":"getting-started.html#initializing-slots"},{"type":"extras","title":"Consuming Streams - Getting Started","doc":"`Tempus.Slots` is optimized for reads, that’s why internally it keeps slots as an `Enum`. It only allows to merge `Stream` into the existing `Tempus.Slots` and the stream will be terminated within the range of the `Enum` already presented. Below is the example of consuming the stream.\n\n```elixir\nholidays = [~D|2020-08-06|, ~D|2020-08-13|]\nweekends = Stream.map([~D|2020-08-08|, ~D|2020-08-20|], & &1)\nschedule = holidays |> Enum.into(%Slots{}) |> Slots.merge(weekends)\n\n#⇒ #Slots<[\n#    #Slot<[\n#      from: ~U[2020-08-06 00:00:00.000000Z],\n#      to: ~U[2020-08-06 23:59:59.999999Z]\n#    ]>,\n#    #Slot<[\n#      from: ~U[2020-08-08 00:00:00.000000Z],\n#      to: ~U[2020-08-08 23:59:59.999999Z]\n#    ]>,\n#    #Slot<[\n#      from: ~U[2020-08-13 00:00:00.000000Z],\n#      to: ~U[2020-08-13 23:59:59.999999Z]\n#    ]>\n#  ]>\n```","ref":"getting-started.html#consuming-streams"},{"type":"extras","title":"Checking State - Getting Started","doc":"With a help of `Tempus.next_busy/2` and `Tempus.next_free/2`, one might get the next busy and/or free slot(s). To calculate the first free slot in 5 business days after `2020-08-05`, the following is to be done.\n\n```elixir\nholidays = [~D|2020-08-07|, ~D|2020-08-13|] # ...\nweekends = [~D|2020-08-08|, ~D|2020-08-09|] # ...\n\nholidays\n|> Enum.into(%Slots{})\n|> Tempus.Slots.merge(weekends)\n|> Tempus.days_ahead(~D|2020-08-05|, 5)\n|> hd()\n#⇒ \"2020-08-14\"\n```","ref":"getting-started.html#checking-state"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}